/*
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#ifndef Edit_included
#define Edit_included


#include <qdialog.h>
#include <qwidget.h>

#include <EditData.h>
#include <barracuda.h>
#include <db.h>
#include <options.h>
#include <Shortcut.h>


#define OPTIONAL 2
#define REQUIRED 1
#define NEXIST   0

#define REF_EMPTY 5
#define ID_COLLISION 4
#define ID_EMPTY 3
#define REF_OK 2
#define REF_WARN 1
#define REF_FALSE 0

#define NR_BUTTONS 6
#define EDIT_OK 0
#define INSERT_OK 1
#define PREV_BUTTON 2
#define NEXT_BUTTON 3
#define CLEAR_BUTTON 4
#define CANCEL_BUTTON 5

class Edit : public QDialog,
	     private EditData
{
    Q_OBJECT

public:
    Edit ( QWidget* parent = NULL, const char* name = NULL);
    
    bool eShow (Database *db, Reference *ref, Options *op);
    bool iShow (Database * db, int ref, Options *op);

private:
    void place (Reference *ref);
    void simplePlace (QWidget *l, int x, int y); //show and place the widget l 
                                            //in place (x,y) in the Editdialog
    void showText ();
    void initCombo (QComboBox *combo);
    void updateCombo (QComboBox *combo, Reference *ref);
    int allRequiredFields (Reference *ref);
    void returnText(int field, QLineEdit *l); // sets the text of l 
    void updateRef (int field, QLineEdit *l);				    

private:
    Shortcut *accel;
    int currentFocus;
    QLineEdit *tabOrder[LAST_FIELD-FIRST_FIELD+2]; 
    
    QPushButton *myButtons[NR_BUTTONS];
    QLineEdit *myFields[(LAST_FIELD - FIRST_FIELD + 1)];
    QLabel *myLabels[(LAST_FIELD - FIRST_FIELD + 1)];

    Database *newDatabase;
    Database *oldDatabase;
    Reference *editRef;

    int index;
    bool inDb;
    long int where; // where in the new database we are
    QString oldId;
    bool nextNew;
    bool edit_insert;
    bool expand;
    bool warnings;
    bool lazyUpdate;

							    
public slots:
      void iok ();
      void eok ();
      void next ();
      void prev ();
      void prevTab ();
      void nextTab ();
      void changed (int index);
      void cancel ();
      void clear ();
      void updateText ();

};

void optional (QLabel *l); //color the label in the optional color
void required (QLabel *l);

void copyRef (Reference *src, Reference *dest);
bool refWarn ();
void idMissing ();
void idDup ();


#endif // Edit_included







