/*
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#ifndef Interface_included
#define Interface_included

#include <qwidget.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qfiledlg.h>
#include <qmsgbox.h>
#include <qtooltip.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>

#include "InterfaceData.h"
#include "MenuBarData.h"
#include "Edit.h"
#include "MultiLabel.h"
#include "MCListBox.h"
#include "Shortcut.h"
#include "keywords.h"
#include "db.h"
#include "barracuda.h"
#include "options.h"
#include "Preferences.h"
#include "ViewTypes.h"
#include "SimpleSearch.h"
#include "PrintDlg.h"
#include "Keybindings.h"
#include "WaitPrint.h"
#include "EditMacros.h"
#include "Sort.h"
#include "Search.h"
#include "FontSelect.h"

#define NORMALDB     0
#define SEARCHRES    1

#define INFO_TIMEOUT 5000

#define POPUP_AUTO_WIDTH     1
#define POPUP_HIDE           2
#define POPUP_FONT           3

class Interface : public QWidget,
		  private InterfaceData,
		  private MenuBarData
		  
{
    Q_OBJECT

public:
    MultiLabel *title;
    MCListBox *table;
    Shortcut *accel;
    
    Interface (QWidget* parent = NULL, const char* name = NULL);
    virtual ~Interface();

    bool setShortcut (int id, int keys[], int n);
    void defaultShortcuts ();
    void setInfoField (const char *text);
    static void decorateButton (QPushButton *, char *, char *);
    void readDatabase (const char *filename, bool show=FALSE);
    void updateFonts ();

protected:
    void resizeEvent (QResizeEvent *);

private:
    void loadBitmaps ();
    void addTips ();
    void connectShortcuts ();
    void connectMenus ();
    void connectBtns ();

    char *columnLabel (int col);

    void setTimedInfo (const char *text);

    void saveDatabase (const char *filename);
    void discardDatabase (bool deleteRef = TRUE);
    bool searchToNormal (bool redisplay = TRUE);
    void setCurrentName (const char *name, bool changeNormal = TRUE);
    void showDatabase (Database *db);
    void showReference (Reference *r);
    MCListBoxItem *makeItem (Reference *r);
    
    void updatePref ();
    void updateShortcuts ();
    void updateFields ();
    void updateTypes ();
    void updateColumns ();

    void initPopup ();

    void doSearch (searchDir dir);
    
    Database *normalDb;
    Database *searchRes;
    int currentView;
    QString currentName;
    QString currentDir;
    QString normalName;
    Options *opt;

    Edit *e;
    EditMacros *em;
    Preferences *p;
    ViewTypes *vt;
    SimpleSearch *ss;
    PrintDlg *pd;
    QTimer *infoTimer;
    Keybindings *k;
    Sort *s;
    Search *sd;
    QString searchString;
    Reference *searchRef;
    bool searchRefTypes[LAST_REFERENCE - FIRST_REFERENCE + 1];
    SortSpec *mySortSpec;
    FontSelect *fs;
    QPopupMenu *popup;
    int chosenCol;
    
public slots:
    void showPopup (int col, int but);
    void chosenPopup (int id);
    void colFont (const QFont &);
    void showEdit ();
    void showInsert ();
    void showInsert (int ref_type);
    void insertArticle ();
    void insertInProceedings ();
    void insertInCollection ();
    void insertInBook ();
    void insertBook ();
    void insertBooklet ();
    void insertPhdThesis ();
    void insertMastersThesis ();
    void insertTechReport ();
    void insertManual ();
    void insertUnPublished ();
    void insertMisc ();
    void insertProceedings ();
    void showEditMacros ();
    void closedMacros (bool);
    void setInfoField (int menuID);
    void clearInfoField ();
    void openDb();
    void openMacros();
    void saveMacros();
    void saveDatabase ();
    void saveAsDb();
    void quit();
    void pref ();
    void types ();
    void saveOptions ();
    void searchBack ();
    void simpleSearch ();
    void searchForward ();
    void newDb ();
    void resizeColumn (int column);
    void print ();
    void rewind ();
    void back ();
    void forward ();
    void fastForward ();
    void keybind ();
    void helpDlg ();
    void about ();
    void sort ();
    void search ();
    void deleteRef ();
    void showNormal ();
    void showSearch ();
    void indexChanged (int oldIndex, int newIndex);
};
#endif // Interface_included

