/* options.h
 *
 * Declares the Options class.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#ifndef OPTIONS_H
#define OPTIONS_H

#include <qfont.h>
#include <keywords.h>
#include <Shortcut.h>
#include <stdlib.h>
#include <stdio.h>

#define MAXSHORTCUT MAXKEYS

struct shortcut
{
    shortcut () { id = 0; n = 0; }
    int id;
    int keys[MAXSHORTCUT];
    int n;
};

typedef struct shortcut shortcut;

class Options
{
public:
    Options (int max_shortcuts);
    ~Options ();
    
    bool setPreference (int pref, int value);
    int getPreference (int pref);
    void setDefaultPref ();
    bool defaultPreference (int pref, int value);

    bool conflicts (shortcut *s1, shortcut *s2);
    bool identical (shortcut *s1, shortcut *s2);
    bool sameKeys (shortcut *s1, shortcut *s2);
    bool setShortcut (shortcut *s);
    bool setShortcut (int id, int keys[], int n);
    QString getShortcutString (int id);
    bool hasShortcut (int id);
    int getFirstDefinedId ();
    bool undefShortcut (shortcut *s);
    bool undefShortcut (int id);
    int getShortcuts (shortcut ***s);
    int maxShortcuts ();
    void setDefaultShortcuts ();
    bool defaultShortcut (shortcut *s);
    shortcut *makeCopy (shortcut *s);

    bool setType (int type, bool enable);
    bool getType (int type);
    void setDefaultTypes ();
    void setDefaultColumns ();
    bool defaultType (int type, bool enable);

    bool setField (int field, bool enable);
    bool getField (int field);
    void setDefaultFields ();
    bool defaultField (int field, bool enable);

    bool setColumnWidth (int type, int width);
    int getColumnWidth (int type);
    void setDefaultWidths ();
    bool defaultWidth (int type, int width);

    bool setColumnFont (int type, QFont f);
    QFont getColumnFont (int type);
    void setDefaultFonts ();
    bool defaultFont (int type, QFont f);

    bool setColumnIndex (int type, int index);
    int columnIndex (int type);
    int columnType (int index);
    int numColumns ();
    void setDefaultIndex ();
    bool defaultIndex (int type, int index);

    void assign (Options *o);

    bool loadFromFile (char *filename);
    void saveToFile (char *filename);

    void print (FILE *fp);
private:
    void initDefaults ();
    
    int preferences[LAST_PREF - FIRST_PREF + 1];
    int defPreferences[LAST_PREF - FIRST_PREF + 1];

    shortcut **shortcuts;
    shortcut **defShortcuts;
    int num_shortcuts;
    int num_defShortcuts;
    int max_shortcuts;

    bool fields[LAST_REFERENCE - FIRST_REFERENCE + 1];
    bool defFields[LAST_REFERENCE - FIRST_REFERENCE + 1];

    bool types[LAST_FIELD - FIRST_FIELD + 2];
    bool defTypes[LAST_FIELD - FIRST_FIELD + 2];

    int idwidth;
    int widths[LAST_FIELD - FIRST_FIELD + 1];
    int defIdwidth;
    int defWidths[LAST_FIELD - FIRST_FIELD + 1];

    int ordering[LAST_FIELD - FIRST_FIELD + 2];
    int defOrdering[LAST_FIELD - FIRST_FIELD + 2];

    QFont fonts[LAST_FIELD - FIRST_FIELD + 2];
    QFont defFonts[LAST_FIELD - FIRST_FIELD + 2];
};
    

#endif
