/*
 * Bib-file scanner.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */


%{

/* C++ declarations */

#include <ctype.h>
#include <string.h>
#include <qstring.h>

typedef struct _semanticValue
{
  int type;
  char *str;
  int line;
} semanticValue;

extern bool lowerIdentifiers;

#define YYSTYPE semanticValue

#define YY_DECL int biblex (YYSTYPE* yylval)
#define YY_NO_UNPUT 1

#include "bib.tab.h"
#include <keywords.h>

static int lookupIdentifier (char* id, int n);
static QString stringToScan;

int currentLine;
int braceLevel;
char closingBrace;
int scanString;
bool findAt;

%}

FIRST          [A-Za-z]
OTHER          [^\"#%'(),=\{\} \t\n]
STRINGLETTER   [^\"\}\{ \t\n]
%x STRING
%x FINDAT

%option noyywrap

%%
    if (findAt)
      {
        BEGIN(FINDAT);
      }
    else if (scanString > 0)
      {
        stringToScan = "";
	braceLevel = 1;
	/* scanString == openingchar=='"' ? 1 : 2. */
	closingBrace = scanString == 1 ? '"' : '}';
        BEGIN(STRING);
      }


<INITIAL,STRING,FINDAT><<EOF>> { return END_OF_FILE; }

<FINDAT>[^\n@]*  { /* eat junk */ }

<FINDAT>\n { currentLine++; }

<FINDAT>@           {    findAt = 0;
                         BEGIN(INITIAL); }

[\t ]                  { /* White space */ }

\n                     { currentLine++; }

[%][^\n]*                   { /* Comment */ }

[Ss][Tt][Rr][Ii][Nn][Gg] { return ATSTRING; }

[0-9]+                 {
  yylval->type = 0;
  yylval->line = currentLine;
  yylval->str = (char*) malloc (sizeof(char)*(bibleng+1));
  strcpy (yylval->str, bibtext);
  return Number;
                       }

{FIRST}{OTHER}*        { 
  yylval->type = lookupIdentifier (bibtext, bibleng);
  yylval->line = currentLine;
  yylval->str = (char*) malloc (sizeof(char)*(bibleng+1));
  strcpy (yylval->str, bibtext);
  return Identifier;
                       }

<STRING>((\\\")|{STRINGLETTER})+ {
  stringToScan += bibtext;
                       }

<STRING>(\})|(\")      {
  /* End of string if braceLevel is 0 */
  if (braceLevel == 0 ||
      (bibtext[bibleng-1] == closingBrace && braceLevel == 1))
    {
      yylval->type = 0;
      yylval->line = currentLine;
      yylval->str = (char*) malloc (sizeof(char)*(stringToScan.length ()+1));
      strcpy (yylval->str, (const char*) stringToScan);
      BEGIN(INITIAL);

      return StringEnd;
    }
  else
    {
      if (bibtext[bibleng-1] == '}')
	braceLevel--;
      stringToScan += bibtext;
    }
                       }

<STRING>\{             {
  braceLevel++;
  stringToScan += bibtext;
                       }

<STRING>[ \t\n]+ {
  int i;
  for (i = 0; i < bibleng; i++)
    if (bibtext[i] == '\n')
      currentLine++;

  stringToScan += ' ';
}

.                      { return bibtext[0];
                         yylval->line = currentLine; }

%%

/* C++ code */

/* lookupIdentifier
 *
 * Args: string, strlen (string)
 * => Keyword number for string (0 if not a keyword)
 */
static int lookupIdentifier (char* id, int n)
{
  int r;

  /* Convert identifier to lower case */
  if (lowerIdentifiers)
    for (r = 0; id[r]; r++)
      id[r] = tolower (id[r]);

  /* Find keyword */
  for (r = 1; keywords[r]; r++)
    if (!strncmp (keywords[r], id, n))
      return r;

  /* Return ''not keyword'' */
  return 0;
}
