/*
 * Edit dialog.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#include <Edit.h>

#include <barracuda.h>
#include <keywords.h>
#include <db.h>
#include <options.h>
#include <Interface.h>

#include <qcombo.h>
#include <qbutton.h>
#include <qcolor.h>
#include <qpalette.h>
#include <qmsgbox.h>
#include <qstring.h>

#define Inherited QDialog

#define PREV_FOCUS_ID 4710
#define CHANGE_FOCUS_ID 4711
#define PREV_ID 4712 
#define NEXT_ID 4713


Edit::Edit
(
	QWidget* parent,
	const char* name
)
	:
	Inherited( parent, name, TRUE ),
	EditData( this )
{
  int keys[2];
  int i = 0;

  connect(eokButton, SIGNAL(clicked()), this, SLOT(eok()));
  connect(iokButton, SIGNAL(clicked()), this, SLOT(iok()));
  connect(nextButton, SIGNAL(clicked()), this, SLOT(next()));
  connect(prevButton, SIGNAL(clicked()), this, SLOT(prev()));
  connect(refCombo, SIGNAL(activated(int)), this, SLOT(changed(int)));
  connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
  connect(clearButton, SIGNAL(clicked()), this, SLOT(clear())); 
  
  myButtons[EDIT_OK] = eokButton;
  myButtons[INSERT_OK] = iokButton;
  myButtons[PREV_BUTTON] = prevButton;
  myButtons[NEXT_BUTTON] = nextButton;
  myButtons[CLEAR_BUTTON] = clearButton;
  myButtons[CANCEL_BUTTON] = cancelButton;

  myFields[(ANNOTE - FIRST_FIELD)] = annoteText;
  myFields[(ADDRESS - FIRST_FIELD)] = adressText;
  myFields[(AUTHOR - FIRST_FIELD)] = authorText;
  myFields[(BOOKTITLE - FIRST_FIELD)] = booktitleText;
  myFields[(CHAPTER - FIRST_FIELD)] = chapterText;
  myFields[(CROSSREF - FIRST_FIELD)] = crossreferenceText;
  myFields[(EDITION - FIRST_FIELD)] = editionText;
  myFields[(EDITOR - FIRST_FIELD)] = editorText;
  myFields[(HOWPUBLISHED - FIRST_FIELD)] = howPublishedText;
  myFields[(INSTITUTION - FIRST_FIELD)] = institutionText;
  myFields[(JOURNAL - FIRST_FIELD)] = journalText;
  myFields[(KEY - FIRST_FIELD)] = keyText;
  myFields[(MONTH - FIRST_FIELD)] = monthText;
  myFields[(NOTE - FIRST_FIELD)] = noteText;
  myFields[(NUMBER - FIRST_FIELD)] = numberText;
  myFields[(ORGANIZATION - FIRST_FIELD)] = organizationText;
  myFields[(PAGES - FIRST_FIELD)] = pagesText;
  myFields[(PUBLISHER - FIRST_FIELD)] = publisherText;
  myFields[(SCHOOL - FIRST_FIELD)] = schoolText;
  myFields[(SERIES - FIRST_FIELD)] = seriesText;
  myFields[(TITLE - FIRST_FIELD)] = titleText;
  myFields[(TYPE - FIRST_FIELD)] = typeText;
  myFields[(VOLUME - FIRST_FIELD)] = volumeText;
  myFields[(YEAR - FIRST_FIELD)] = yearText;
  myFields[(ID - FIRST_FIELD)] = idText;

  myLabels[(ANNOTE - FIRST_FIELD)] = annote;
  myLabels[(ADDRESS - FIRST_FIELD)] = adress;
  myLabels[(AUTHOR - FIRST_FIELD)] = author;
  myLabels[(BOOKTITLE - FIRST_FIELD)] = booktitle;
  myLabels[(CHAPTER - FIRST_FIELD)] = chapter;
  myLabels[(CROSSREF - FIRST_FIELD)] = crossreference;
  myLabels[(EDITION - FIRST_FIELD)] = edition;
  myLabels[(EDITOR - FIRST_FIELD)] = editor;
  myLabels[(HOWPUBLISHED - FIRST_FIELD)] = howPublished;
  myLabels[(INSTITUTION - FIRST_FIELD)] = institution;
  myLabels[(JOURNAL - FIRST_FIELD)] = journal;
  myLabels[(KEY - FIRST_FIELD)] = key;
  myLabels[(MONTH - FIRST_FIELD)] = month;
  myLabels[(NOTE - FIRST_FIELD)] = note;
  myLabels[(NUMBER - FIRST_FIELD)] = number;
  myLabels[(ORGANIZATION - FIRST_FIELD)] = organization;
  myLabels[(PAGES - FIRST_FIELD)] = pages;
  myLabels[(PUBLISHER - FIRST_FIELD)] = publisher;
  myLabels[(SCHOOL - FIRST_FIELD)] = school;
  myLabels[(SERIES - FIRST_FIELD)] = series;
  myLabels[(TITLE - FIRST_FIELD)] = title;
  myLabels[(TYPE - FIRST_FIELD)] = type;
  myLabels[(VOLUME - FIRST_FIELD)] = volume;
  myLabels[(YEAR - FIRST_FIELD)] = year;
  myLabels[(ID - FIRST_FIELD)] = id;

  for (i = 0; i < (LAST_FIELD - FIRST_FIELD + 1); i++)
    {
      connect(myFields[i], SIGNAL(returnPressed()), 
	      this, SLOT(updateText()));
      myFields[i]->setFocusPolicy (ClickFocus);
    }

  for (i = 0; i < (NR_BUTTONS); i++)
    {
      myButtons[i]->setFocusPolicy (ClickFocus);
    }

  initCombo(refCombo);
  refCombo->setFocusPolicy (ClickFocus);
  
  index = 0;
  inDb = FALSE;
  where = 0;
  oldId = "";
  edit_insert = FALSE;
  expand = FALSE;
  warnings = FALSE;
  lazyUpdate = FALSE;

  accel = new Shortcut(this);
  currentFocus = 0;
  tabOrder[0] = NULL;
  keys[0] = SHIFT + Key_Backtab;
  accel->insertItem (keys, 1, PREV_FOCUS_ID);
  accel->connectItem (PREV_FOCUS_ID, this, SLOT (prevTab()));
    keys[0] = Key_Tab;
  accel->insertItem (keys, 1, CHANGE_FOCUS_ID);
  accel->connectItem (CHANGE_FOCUS_ID, this, SLOT (nextTab()));
  keys[0] = Key_Next;
  accel->insertItem (keys, 1, NEXT_ID);
  accel->connectItem (NEXT_ID, this, SLOT (next ()));
  keys[0] = Key_Prior;
  accel->insertItem (keys, 1, PREV_ID);
  accel->connectItem (PREV_ID, this, SLOT (prev ()));
}


/* allRequiredFields
 * 
 * Args: Type   - which is the integer that defines a reference
 *
 * => bool, TRUE if all required fields are filled, else FALSE
 */

int Edit::allRequiredFields (Reference *ref)
{
  int i = 0;
  long int crashIndex = -1;

  int status = REF_OK;
  int warns = 0;
  int numberOfFields = 0;
  MacroOrString *mos;

  if (!edit_insert)
    oldId = ref->id;
  
  for (i=FIRST_FIELD; i<=LAST_FIELD; i++)
    {
      if (ref->hasField (i) == REQUIRED) // counts the number of required fieds
	{
	  numberOfFields++;
	};

      // counts the number of empty required fields
      if ((ref->hasField (i) == REQUIRED) && 
	  (((ref->getField (i))->getString () == NULL) ||
	   (!strcmp ((const char*)((ref->getField (i))->getString ()),
		     ""))))
	{
	  warns++;
	}
    }
  if(!edit_insert)
    {
      if (warns == numberOfFields)
	{
	  status = REF_EMPTY;  // we belive that ref is empty
	}
      else
	{
	  if (warns != 0)
	    status = REF_WARN;
	  else
	    status = REF_OK;
	}
    }
  else
    {
      if (warns > 0)
	status = REF_WARN;
    }
	
  QString str(ref->id);
  str = (ref->id).stripWhiteSpace ();

  if (status == REF_EMPTY)
    {
      if (!(str.isEmpty ()))
	status = REF_WARN;
      
      for (i=FIRST_FIELD; i<=LAST_FIELD-FIRST_FIELD; i++)
	{
	  if (ref->hasField (i) == OPTIONAL)
	    {
	      mos = ref->getField (i);
	      if ((mos->getString () != NULL) &&
		  (strcmp(mos->getString (), "") != 0))
		{	
		  status = REF_WARN;
		  break;
		}
	    }
	}
    }  
  
  if ((ref->id == NULL) && // id NULL illegal, but legal if everything is
      (status != REF_EMPTY))   // empty
    status = ID_EMPTY;
  
  if ((str.isEmpty ()) && // id is just whitespace illegal, but legal if 
      (status != REF_EMPTY))
    status = ID_EMPTY;      // every thing is empty
  
  // check for collisions in old database
  // -1 stands for no collision
  if ((oldDatabase->idCollision (ref->id)) != -1) 
    {
      if ((edit_insert) && // if we are doing some editing and the oldId
	  (oldId != ref->id))
	status = ID_COLLISION;
       
      if (!edit_insert)
	status = ID_COLLISION;
    }

  // check for collisions in new database
  if (!edit_insert) // doing some inserting
    {  //-1 stands for no crash
      crashIndex = newDatabase->idCollision (ref->id);
      if (crashIndex != -1) 
	if ((crashIndex != where) ||
	    ((crashIndex == where) && // if last inserted and this one have
	     (!inDb)))                // the same id & this one can't be in
	  status = ID_COLLISION;      // newDatabase
    }
  
  return status;
}

/* eShow
 *
 * Args: ref    - the reference which to edit
 *
 * => nothing, displays the editdialog with apropriate fields
 */
 
bool Edit::eShow (Database *db, Reference *ref, Options *op)
{
  oldDatabase = db;
  edit_insert = TRUE; //if the dialog is a edit or an insert dialog

  expand = op->getPreference (EXPAND_MACRO);  //if prefs. is to expand macros  
  warnings = (!(op->getPreference (NO_SILLY_WARN))); 
  lazyUpdate = (op->getPreference (LAZY_COLUMN));
  //if the user wants to see warnings and lazy update

  if (accel->isItemEnabled (NEXT_ID))
    accel->setItemEnabled (NEXT_ID, FALSE);
  
  if (accel->isItemEnabled (PREV_ID))
    accel->setItemEnabled (PREV_ID, FALSE);
  
  if (lazyUpdate)
    {
      disconnect (refCombo, 0, this, 0);
      connect (refCombo, SIGNAL (activated (int)), this, SLOT (changed (int)));
    }
  else
    {
      disconnect (refCombo, 0, this, 0);
      connect (refCombo, SIGNAL (highlighted (int)), 
	       this, SLOT (changed (int)));
    }
  
  editRef = new Reference (ref->getType ());

  if(editRef == NULL)
    {
      return FALSE;
    }

  editRef->id = (const char*) ref->id; // must be done with brute force
  oldId = ref->id; // to control make sure that if the id  
  
  copyRef (ref, editRef);   // copying, deep of ref 

  this->setCaption ("Editing");
  this->updateCombo (refCombo, editRef); // updates the ComboBox
  this->place (editRef); // place all neccesary labels and lineedit
  
  this->showText (); // show the reference textfields
  this->show ();

  // change if we get result () == TRUE user made som changes
  
  if(result())
    {  
      ref->id = (const char*) editRef->id;
      copyRef (editRef, ref);
    }

  delete editRef;

  return result ();
}

/* iShow
 *
 * Args: ref    - which is the reference which the user wants to start to to 
 *                the old database
 * 
 * => newDatabase, which is the database with new references
 */

bool Edit::iShow(Database *db, int ref, Options *op)
{
  int i = 0;
  where = -1;
  oldDatabase = db;
  newDatabase = new Database (20);
  edit_insert = FALSE;
  inDb = FALSE;
  
  expand = op->getPreference (EXPAND_MACRO);  //if prefs. is to expand macros  
  warnings = (!(op->getPreference (NO_SILLY_WARN))); 
  lazyUpdate = (op->getPreference (LAZY_COLUMN));
  //if the user wants to see warnings and lazy update

  if (!(accel->isItemEnabled (NEXT_ID)))
    accel->setItemEnabled (NEXT_ID, TRUE);
      
  if (!(accel->isItemEnabled (PREV_ID)))
    accel->setItemEnabled (PREV_ID, TRUE);

  if (lazyUpdate)
    {
      disconnect (refCombo);
      connect (refCombo, SIGNAL (activated (int)), this, SLOT (changed (int)));
    }
  else
    {
      disconnect (refCombo);
      connect (refCombo, SIGNAL (highlighted (int)), 
	       this, SLOT (changed (int)));
    };

  editRef = new Reference (ref);
  editRef->setType (ref);

  for (i=FIRST_FIELD; i<=LAST_FIELD; i++)
    {
      editRef->setField(i, "", STRING, TRUE);
    }
  
  this->setCaption ("Inserting");
  this->place (editRef); // place all neccesary labels and lineedits
  this->showText ();
  this->updateCombo (refCombo, editRef); // updates refCombo
  
  this->show ();
    
  bool answer = result ();
  // remove newDatabase but, not the references in it
  delete newDatabase;
  
  return answer;
};


/* eok
 *
 * Args: none
 *
 * => nothing, checks if the refrence is OK 
 */

void Edit::eok ()
{
  int referenceOk = REF_OK;

  updateText ();
  referenceOk = allRequiredFields (editRef);
  
  if (referenceOk == REF_OK)
      this->done (TRUE);
  
  if (referenceOk == ID_EMPTY)
    idMissing ();
  
  if (referenceOk == ID_COLLISION)
    idDup ();
  
  if (referenceOk == REF_WARN)
    {
      if (warnings)
	{
	  if(refWarn ())
	    this->done (TRUE);
	}
      else
	{
	  this->done (TRUE);
	}
    }
}

/* iok
 * 
 * Args: none
 *
 * => nothing, checks if the temporal database is ok
 */

void Edit::iok()
{
  int referenceOk = REF_OK;

  updateText ();
  referenceOk = allRequiredFields (editRef);
  
  if (inDb)
    {
      if (referenceOk == REF_OK)
	{
	  copyRef (editRef, (*newDatabase)[where]);
	  oldDatabase->merge (*newDatabase);
	  delete editRef;
	  this->done (TRUE);
	}
      
      if (referenceOk == REF_WARN)
	{
	  if (warnings)
	    {
	      if (refWarn ())
		{
		  copyRef (editRef, (*newDatabase)[where]);
		  oldDatabase->merge ((*newDatabase));
		  delete editRef;
		  this->done (TRUE);
		}
	    }
	  else
	    {
	      copyRef (editRef, (*newDatabase)[where]);
	      oldDatabase->merge ((*newDatabase));
	      delete editRef;
	      this->done (TRUE);
	    }
	}

      if (referenceOk == REF_EMPTY)
	{
	  idMissing ();
	}
      
      if (referenceOk == ID_COLLISION)
	{
	  idDup ();
	}

      if (referenceOk == ID_EMPTY)
	{
	  idMissing ();
	}
    }
  else
    {
      if (referenceOk == REF_OK)
	{
	  newDatabase->addReference (editRef);
	  oldDatabase->merge ((*newDatabase));
	  this->done (TRUE);
	}

      if (referenceOk == REF_WARN)
	{
	  if (warnings)
	    {
	      if (refWarn ())
		{
		  newDatabase->addReference(editRef);
		  oldDatabase->merge((*newDatabase));
		  this->done (TRUE);
		}
	    }
	  else
	    {
	      newDatabase->addReference (editRef);
	      oldDatabase->merge (*newDatabase);
	      this->done (TRUE);
	    }
	}
      
      if (referenceOk == REF_EMPTY)
	{
	  delete editRef;
	  oldDatabase->merge (*newDatabase);
	  this->done (TRUE);
	}
      
      if (referenceOk == ID_EMPTY)
	idMissing ();
      
      if (referenceOk == ID_COLLISION)
	idDup ();
    } 
}


/* next
 *
 * Args: none
 *
 * => nothing, displays the next reference in the temporal database
 */

void Edit::next ()
{
  int referenceOk = REF_OK;

  updateText ();
  referenceOk = allRequiredFields (editRef);

  if (inDb)
    {
      if (referenceOk == REF_OK) 
	{
	  copyRef (editRef, (*newDatabase)[where]);
	  where++;
	  if (where > newDatabase->maxIndex ())
	    {
	      editRef = new Reference (editRef->getType ());
	      where = newDatabase->maxIndex ();
	      inDb = FALSE;
	    }
	  else
	    {
	      copyRef((*newDatabase)[where],editRef);
	    }
	}
      
      if (referenceOk == REF_WARN)
	{
	  if (warnings)
	    {
	      if (refWarn ())
		{
		  copyRef (editRef, (*newDatabase)[where]);
		  where++;
		  if (where > newDatabase->maxIndex ())
		    {
		      editRef = new Reference (editRef->getType ());
		      where = newDatabase->maxIndex ();
		      inDb = FALSE;
		    }
		  else
		    {
		      copyRef((*newDatabase)[where],editRef);
		    }
		}
	    }
	  else
	    {
	      copyRef (editRef, (*newDatabase)[where]);
	      where++;
	      if (where > newDatabase->maxIndex ())
		{
		  editRef = new Reference (editRef->getType ());
		  where = newDatabase->maxIndex ();
		  inDb = FALSE;
		}
	      else
		{
		  copyRef((*newDatabase)[where],editRef);
		}
	    }
	}

      if (warnings)
	{
	  if ((referenceOk == REF_EMPTY) ||
	      (referenceOk == ID_EMPTY))
	    {
	      idMissing ();
	    }
	}
      
      if (referenceOk == ID_COLLISION)
	{
	  idDup ();
	}      
	
    }
  else
    {
      if (referenceOk == REF_OK)
	{
	  newDatabase->addReference (editRef);
	  where++;
	  editRef = new Reference (editRef->getType ());
	  inDb = FALSE;
	}
      
      if (referenceOk == REF_WARN)
	{
	  if (warnings)
	    {
	      if (refWarn ())
		{
		  newDatabase->addReference (editRef);
		  where++;
		  editRef = new Reference (editRef->getType ());
		  inDb = FALSE;
		}
	    }
	  else
	    {
	        newDatabase->addReference (editRef);
		where++;
		editRef = new Reference (editRef->getType ());
		inDb = FALSE;
	    }
	}
      
      if ((referenceOk == REF_EMPTY) ||
	  (referenceOk == ID_EMPTY))
	{
	  idMissing ();
	}
      
      if (referenceOk == ID_COLLISION)
	{
	  idDup ();
	}
    }

  this->updateCombo (refCombo, editRef);
  this->place (editRef);
  this->showText ();
};

/* prev
 *
 * Args: none
 * 
 * => nothing, displays the previous reference in the temporal database
 */

void Edit::prev ()
{
  int referenceOk = REF_OK;
  updateText ();
  
  referenceOk = allRequiredFields (editRef);
  
  if (inDb) // if editRef is a copy of a allready existing reference
    {
      if (referenceOk == REF_OK) 
	{
	  if (where == 0)
	    {
	      copyRef (editRef, (*newDatabase)[where]);
	    }
	  else
	    {
	      copyRef (editRef, (*newDatabase)[where]);
	      where--;
	      copyRef((*newDatabase)[where], editRef);
	    }
	}
      
      if (referenceOk == REF_WARN) // a 'required' field is missing
	{
	  if (warnings)
	    {
	      if (refWarn ())
		{
		  if (where == 0)
		    {
		      copyRef (editRef, (*newDatabase)[where]);
		    }
		  else
		    {
		      copyRef (editRef, (*newDatabase)[where]);
		      where--;
		      copyRef((*newDatabase)[where], editRef);
		    }
		}
	    }
	  else
	    {
	      if (where == 0)
		{
		  copyRef (editRef, (*newDatabase)[where]);
		}
	      else
		{
		  copyRef (editRef, (*newDatabase)[where]);
		  where--;
		  copyRef ((*newDatabase)[where], editRef);
		}
	    }
	}
      
      if ((referenceOk == REF_EMPTY) ||
	  (referenceOk == ID_EMPTY))
	{
	  idMissing ();
	}

      if (referenceOk == ID_COLLISION)
	{
	  idDup ();
	}

      inDb = TRUE;
    }
  else // editRef is not in the newDatabase
    {
      if (referenceOk == REF_OK)
	{
	  newDatabase->addReference (editRef);
	  editRef = new Reference (editRef->getType ());
	  where++;
	  if (where == 0)
	    {
	      copyRef ((*newDatabase)[where], editRef);
	    }
	  else
	    {
	      where--;
	      copyRef ((*newDatabase)[where], editRef);
	    }
	}

      if (referenceOk == REF_WARN)
	{
	  if (warnings)
	    {
	      if (refWarn ())
		{
		  newDatabase->addReference (editRef);
		  editRef = new Reference (editRef->getType ());
		  where ++;
		  if (where == 0)
		    {
		      copyRef ((*newDatabase)[where], editRef);
		    }
		  else
		    {
		      where--;
		      copyRef ((*newDatabase)[where], editRef);
		    }
		}
	    }
	  else
	    {
	      newDatabase->addReference (editRef);
	      editRef = new Reference (editRef->getType ());
	      where++;
	      if (where == 0)
		{
		  copyRef ((*newDatabase)[where], editRef);
		}
	      else
		{
		  where--;
		  copyRef ((*newDatabase)[where], editRef);
		}
	    }
	}

      if (referenceOk == REF_EMPTY)
	{
	  if (where == newDatabase->maxIndex ()) // only if we are at the 
	    {                                 // last ref in database
	      // can we go back when editRef is empty
	      if(where > -1)
		copyRef ((*newDatabase)[where], editRef);
	    }
	  else
	    {
	      idMissing ();
	    }
	}

      if (referenceOk == ID_EMPTY)
	{
	  idMissing ();
	}

      if (referenceOk == ID_COLLISION)
	{
	  idDup ();
	}
      
      if ((referenceOk == ID_COLLISION) || 
	  (referenceOk == ID_EMPTY) ||
	  (where < 0))
	{
	  inDb = FALSE;
	}
      else
	{
	  inDb = TRUE;
	};
    }

  this->updateCombo (refCombo, editRef);
  this->place (editRef);
  this->showText ();
}


/* prevTab
 *
 * Changes focus to the previous edit line.
 */

void Edit::prevTab ()
{
  int i;
  
  for (i = 0; tabOrder[i] != NULL; i++)
    if (tabOrder[i]->hasFocus ())
	currentFocus = i;

  if (currentFocus == 0)
  {
      tabOrder[i-1]->setFocus ();
      currentFocus = i-1;
  }
  else
  {
      currentFocus--;
      tabOrder[currentFocus]->setFocus ();
  }
  updateText ();
}

/* nextTab
 *
 * Changes focus to the next edit line.
 */

void Edit::nextTab ()
{
  int i;
  
  for (i = 0; tabOrder[i] != NULL; i++)
    if (tabOrder[i]->hasFocus ())
      {
	currentFocus = i;
	break;
      }

  if (tabOrder[currentFocus + 1] == NULL)
    {
      tabOrder[0]->setFocus ();
      currentFocus = 0;
    }
  else
    {
      currentFocus++;
      tabOrder[currentFocus]->setFocus ();
    }
  updateText ();
}


/* cancel
 *
 * Args: none
 *
 * => nothing, close edit and free editDatabase and all references in it
 */

void Edit::cancel ()
{
  if(!edit_insert)
    {
      newDatabase->removeAll(TRUE);
    }
  this->done (FALSE);
}

/* clear
 *
 * Args: none
 * 
 * => nothing, clear all QLineEdit
 */

void Edit::clear ()
{
  int i=0;

  editRef->id = ""; 
  //kolla expand mode?
    for (i = FIRST_FIELD; i <= LAST_FIELD; i++)
    {
      if(editRef->hasField (i) != NEXIST)
	editRef->setField (i, "", STRING, TRUE);  
    };
  
  this->showText ();
  this->show ();
}


/////////////////////////////////////////////////////////////////////////////

/* returnText
 *
 * Args: field   - which is the index of which field you want returned
 *
 * => const char*, which is depending on the expand-mode a macro, a expanded
 *                 macro or a string
 */

void Edit::returnText (int field, QLineEdit* l)
{
  MacroOrString *macroStr;

  macroStr = editRef->getField (field);
  if (macroStr != NULL) // i.e. ref does have a field f
    {
      if (lookupMacro (macroStr->getMacro ()) == NULL) // not a macro
	{
	  QString qStr (macroStr->getString ());
	  if (qStr.contains ('#'))
	    {// make l->text bold to show that it contains MACROS
	      QFont f (QFont::defaultFont ());
	      f.setBold (TRUE);
	      l->setFont (f);
	    }
	  else 
	    {// makes l->text normal to show that it's a STRING
	      QFont f (QFont::defaultFont ());
	      f.setBold (FALSE);
	      l->setFont (f);
	    };
	}
      else
	{
	  // makes l-text normal to show that it contains a MACRO
	  QFont f ("Times", 12, QFont::Bold);
	  l->setFont (f);
	};
      if (expand) // kolla om expand-mode on || off
	{
	  l->setText (macroStr->getString ());//expand-mode on
	}
      else
	{
	  l->setText (macroStr->getMacro ()); //expand-mode off
	}
    }
};

/* showText
 *
 * Args: ref    - the reference which to edit
 *
 * => nothing, displays those string which already exist in the reference
 */

void Edit::showText ()
{
  int i = 0;

  idText->setText (editRef->id);

  for (i = FIRST_FIELD; i < (LAST_FIELD + 1); i++)
    {
      if (i != ID)
	{
	  returnText (i , myFields[(i-FIRST_FIELD)]);
	}
    }
}

/* updateRef
 *
 * Args: field  - which is a int which specifies which field to update
 *       l      - which is a QLineEdit* which points at the QLineEdit which 
 *                contains the text
 *
 * => nothing, just makes a update in the reference
 */

void Edit::updateRef (int field, QLineEdit* l)
{
  if (editRef->hasField (field) != NEXIST)
    {
      QString qStr(l->text ());
      if (expand)
	{
	  if (strcmp((editRef->getField (field))->getString (),
		     l->text ()) != 0) // they are diffrent
	    {
	      if (lookupMacro (l->text ()) == NULL)
		{
		  if (qStr.contains ('#'))
		    {
		      editRef->setField (field, (char*) l->text (), 
					 CATENATED, TRUE);
		    }
		  else
		    {
		      editRef->setField (field, (char*) l->text (), 
					 STRING, TRUE);
		    }
		}
	      else
		{
		  editRef->setField (field, (char*) l->text (), MACRO, TRUE);
		}
	    }
	}
      else
	{
	  if (strcmp (((editRef->getField (field))->getMacro ()),
		      l->text ()) != 0) // they are diffrent
	      {
		if (lookupMacro (l->text ()) == NULL)
		    {
		      if (qStr.contains ('#'))
			{
			  editRef->setField (field, (char*) l->text (), 
					     CATENATED, TRUE);
			}
		      else
			{
			  editRef->setField (field, (char*) l->text (),
					     STRING, TRUE);
			}
		    }
		else
		  {
		    editRef->setField (field, (char*) l->text (), MACRO, TRUE);
		  }
	      }
	}
    }
}

/* updateText
 *
 * Args: none
 *
 * => nonthing, just changes in editRef
 */

void Edit::updateText ()
{
  int i = 0;
  editRef->id = idText->text ();
  
  for (i = FIRST_FIELD; i < (LAST_FIELD + 1); i++)
    {
      if (i != ID) //id is allreday updated above
	{
	  updateRef (i, myFields[(i - FIRST_FIELD)]);
	}
    }
  
  showText();
}


/* place
 *
 * Args: nothing
 *
 * => nothing, place labels and inputfields on the right place
 */

void Edit::place (Reference *ref)
{
  int i = 0;
  int rc = 0; // counts how many fields there are in ref 
  int x1 = 20; 
  int y1 = 10; // x1 and y1 just to get a nice alignment 
  int x = 0;
  int y = 0;// x and y is the coordinates to place a widget on the dialog 
  currentFocus = 0;

  x=x1;
  y=y1;

  simplePlace (requiredLabel, x, y);
  rc++;
  y=y1+20*rc;
  
  simplePlace (id, x, y);
  simplePlace (idText, x+100, y);
  rc++;
  tabOrder[currentFocus++] = idText;

  for (i = 0; i < NR_BUTTONS; i++)
    {
      myButtons[i]->hide();
    }

  for (i = FIRST_FIELD; i <= LAST_FIELD; i++)
    {
      myFields[(i-FIRST_FIELD)]->hide ();
      myLabels[(i-FIRST_FIELD)]->hide ();
    }

  for (i = FIRST_FIELD; i <= LAST_FIELD; i++)
    {
      if (ref->hasField (i) == REQUIRED)
	{
	  tabOrder[currentFocus++] = myFields[(i-FIRST_FIELD)];
	  required (myLabels [(i-FIRST_FIELD)]);
	  y=y1+20*rc;
	  simplePlace (myLabels[(i-FIRST_FIELD)], x, y);
	  simplePlace (myFields[(i-FIRST_FIELD)], x+100, y);
	  rc++;
	}
    }
  
  rc++; /* add some distance between required and optional fields */
  optional (optionalLabel);
  y=y1+20*rc;
  simplePlace (optionalLabel, x, y);
  rc++;

  for(i = FIRST_FIELD; i <= (LAST_FIELD); i++)
    {
      if (ref->hasField (i) == OPTIONAL)
	{
	  tabOrder[currentFocus++] = myFields[(i-FIRST_FIELD)];
	  optional (myLabels [(i-FIRST_FIELD)]);
	  y=y1+20*rc;
	  simplePlace (myLabels[(i-FIRST_FIELD)], x, y);
	  simplePlace (myFields[(i-FIRST_FIELD)], x+100, y);
	  rc++;
	}
    }
  
  rc = rc+2;

  tabOrder[currentFocus] = NULL;
  currentFocus = 0;

  x = 100+10+390+20;
  y = rc*20+50;

  resize(x, y);
  
  this->setMinimumSize(x, y); // to inhibits the user to resize the dialog
  this->setMaximumSize(x, y);

  if(edit_insert) // if the editDialog in edit-mode
    {
      y = y1+20*rc;
      x = x1;
      simplePlace(eokButton, x, y);
    }
  else  // if the editDialog in insert-mode
    {
      y = y1+20*rc;
      x = x1;
      simplePlace(iokButton, x, y);

      x = x+60+20;
      simplePlace(prevButton, x, y);
      x = x+40;
      simplePlace(nextButton, x, y);
      Interface::decorateButton (prevButton, "<", "/back.bmp");
      Interface::decorateButton (nextButton, ">", "/play.bmp");
    };
  // the next- and prev-buttons will be placed 20 pixels to the right of the 
  // OK-button
  
  x = ((this->minimumSize ()).width ());
  x = (int)((x - ((refCombo-> size ()).width ()))/2);

  simplePlace(refCombo, x, y);
  
  x = ((this->minimumSize ()).width ());

  x = x - ((cancelButton->size ()).width ()) - 10;
  simplePlace(cancelButton, x, y);
 
  x = x - ((clearButton->size ()).width ()) - 10;
  simplePlace(clearButton, x, y);

  idText->setFocus ();
}

/* initCombo
 * 
 * Args: combo - which is a pointer to the QComboBox to initiate
 *
 * => nothing, just fills combo with the diffrent reference
 */

void Edit::initCombo (QComboBox *combo)
{
  const char *refs[] = { "Article", "Book", "Booklet", "InBook", "InCollection", "InProceedings", "Manual", "MasterThesis", "Misc.", "Ph.D.Thesis", "Proceeding", "TechReport", "Unpublished", 0 };
  combo->insertStrList(refs);
}

/* updateCombo
 *
 * Args: combo - which is a pointer to the QComboBox to update
 *       ref   - which is a reference which type combo should be updated to
 *
 * => nothing, just changes combo
 */

void Edit::updateCombo (QComboBox *combo, Reference *ref)
{
  switch (ref->getType ())
    {
    case ARTICLE:
      combo->setCurrentItem (0);
      break;
    case BOOK:
      combo->setCurrentItem (1);
      break;
    case BOOKLET:
      combo->setCurrentItem (2);
      break;
    case INBOOK:
      combo->setCurrentItem (3);
      break;
    case INCOLLECTION:
      combo->setCurrentItem (4);
      break;
    case INPROCEEDINGS:
      combo->setCurrentItem (5);
      break;
    case MANUAL:
      combo->setCurrentItem (6);
      break;
    case MASTERSTHESIS:
      combo->setCurrentItem (7);
      break;
    case MISC:
      combo->setCurrentItem (8);
      break;
    case PHDTHESIS:
      combo->setCurrentItem (9);
      break;
    case PROCEEDINGS:
      combo->setCurrentItem (10);
      break;
    case TECHREPORT:
      combo->setCurrentItem (11);
      break;
    case UNPUBLISHED:
      combo->setCurrentItem (12);
      break;
    default:
      combo->setCurrentItem (12);
      break;
    }
};

/* changed
 *
 * Args: index   - which is the index in refCombo which the editdialog should
 *                 be changed to
 *
 * => nothing, change the editdialog
 */

void Edit::changed (int index)
{
  switch(index)
    {
    case 0:
      editRef->setType (ARTICLE);
      break;
    case 1:
      editRef->setType (BOOK);
      break;
    case 2:
      editRef->setType (BOOKLET);
      break;
    case 3:
      editRef->setType (INBOOK);
      break;
    case 4:
      editRef->setType (INCOLLECTION);
      break;
    case 5:
      editRef->setType (INPROCEEDINGS);
      break;
    case 6:
      editRef->setType (MANUAL);
      break;
    case 7:
      editRef->setType (MASTERSTHESIS);
      break;
    case 8:
      editRef->setType (MISC);
      break;
    case 9:
      editRef->setType (PHDTHESIS);
      break;
    case 10:
      editRef->setType (PROCEEDINGS);
      break;
    case 11:
      editRef->setType (TECHREPORT);
      break;
    case 12:
      editRef->setType (UNPUBLISHED);
      break;
    default:
      editRef->setType (UNPUBLISHED);
      break;
    }
    
  this->updateText ();  
  this->place (editRef);
};

/* simplePlace */

void Edit::simplePlace (QWidget *l, int x, int y)
{
  l->move (x, y);
  l->show ();
}

/* optional
 * 
 * Args: l    - which is the QLabel to color in optional color
 *
 * => nothing, just changes the label color
 */

void optional(QLabel* l)
{
  QColorGroup normal(QColor("grey"), QColor("grey"), 
		     QColor("lightgrey"), QColor("darkgrey"),
		     QColor("grey"), QColor("white"), 
		     QColor("grey"));
  QColorGroup disabled;
  QColorGroup active;
  QPalette palette( normal, disabled, active );
  l->setPalette( palette );
}

/* required
 *
 * Args: l   _ which is the label to be colored in the required color
 *
 * => nothing, just color the label in black
 */

void required (QLabel *l)
{
  QColorGroup normal (QColor ("grey"), QColor ("grey"), 
		      QColor ("lightgrey"), QColor ("darkgrey"),
		      QColor ("grey"), QColor ("black"), 
		      QColor ("grey"));
  QColorGroup disabled;
  QColorGroup active;
  QPalette palette (normal, disabled, active);
  l->setPalette (palette);
}

/* copyRef */

void copyRef(Reference *src, Reference *dest)
{
  *dest = *src;
  dest->setType (src->getType ());
}

/* refWarn */

bool refWarn()
{
  bool warnD = QMessageBox::query ("Warning", 
				   "Not all required fields have been filled.
\nContinue?",
				   "Yes",
				   "No");
  return warnD;
}

/* idMissing */

void idMissing()
{
  QMessageBox::message ("Warning", 
			"You haven't filled in the Id field.");
}

/* idDup */

void idDup ()
{
  QMessageBox::message ("Warning", 
			"This Id already exist.");
}
