/*
 * Edit macros dialog.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#include "EditMacros.h"
#include <qpen.h>

#define Inherited QWidget

EditMacros::EditMacros
(
	QWidget* parent,
	const char* name
)
	:
	Inherited( parent, name ),
	EditMacrosData( this )
{
	setCaption( "Edit Macros" );

	table = new MCListBox (this);
	title = new MultiLabel (this);

	QObject::connect (title, SIGNAL (widthChanged (int, int)),
			  table, SLOT (setColumnWidth (int, int)));
	QObject::connect (table, SIGNAL (horizValueChanged (int)),
			  title, SLOT (setScrollOffset (int)));
	QObject::connect (title, SIGNAL (sepDblClicked (int)),
			  this, SLOT (resizeColumn (int)));
	QObject::connect (table, SIGNAL (highlighted (int)),
			  this, SLOT (displayMacro (int)));

	QObject::connect (idEdit, SIGNAL (textChanged (const char *)),
			  this, SLOT (edited (const char *)));
	QObject::connect (expandEdit, SIGNAL (textChanged (const char *)),
			  this, SLOT (edited (const char *)));
			  

	table->setGeometry (10, 40, 340, 170);
	table->setHorizTracking (TRUE);
	table->setDrawLast (TRUE);
	table->insertColumn ();
	table->insertColumn ();

	title->setGeometry (11, 10, 339, 25);
	title->setFrameStyle (QFrame::Panel | QFrame::Raised);
	title->setDrawLast (TRUE);
	title->insertColumn ("Macro id");
	title->insertColumn ("Expanded macro");
	title->setColumnWidth (140, 0);
	title->setColumnWidth (198, 1);

	setMinimumSize (440, 320);

	connect (okBtn, SIGNAL (clicked ()), this, SLOT (ok ()));
	connect (cancelBtn, SIGNAL (clicked ()), this, SLOT (cancel ()));
	connect (insertBtn, SIGNAL (clicked ()), this, SLOT (insert ()));
	connect (deleteBtn, SIGNAL (clicked ()), this, SLOT (del ()));
	connect (clearBtn, SIGNAL (clicked ()), this, SLOT (clear ()));
	
	insertBtn->setEnabled (FALSE);
}


EditMacros::~EditMacros()
{
    delete table;
    delete title;
}


/* resizeEvent
 *
 * Resizes the window
 */

void EditMacros::resizeEvent (QResizeEvent *)
{
    int lbw = width() - 100;
    int lbh = height() - 150;
    int leftRowX = width() - 80;
    int bottomRowY = height() - 40;
    int idy = table->y() + lbh + 10;

    title->resize (lbw, 25);
    table->resize (lbw, lbh);

    idLabel->move (idLabel->x(), idy);
    expandLabel->move (expandLabel->x(), idy+20);

    idEdit->setGeometry (idEdit->x(), idy, lbw-100, idEdit->height());
    expandEdit->setGeometry (expandEdit->x(), idy+20, 
			    lbw-100, expandEdit->height());

    okBtn->move (okBtn->x(), bottomRowY);
    cancelBtn->move (table->x()+table->width()-cancelBtn->width(), bottomRowY);

    insertBtn->move (leftRowX, insertBtn->y());
    deleteBtn->move (leftRowX, deleteBtn->y());
    clearBtn->move (leftRowX, clearBtn->y());
}

/* eShow
 *
 * Opens the window and displays all macros.
 */

void EditMacros::eShow (Options *o)
{
    opt = o;
    showMacros ();
    insertBtn->setEnabled (FALSE);
    show ();
}


void EditMacros::showMacros ()
{
    id_entry_t *m;

    idEdit->setText ("");
    expandEdit->setText ("");
    table->setDelimiterStyle ((enum PenStyle)
			      opt->getPreference (DEL_STYLE));
    table->setAutoUpdate (FALSE);
    table->clear (FALSE);
    for (m = sts_current_scope()->all; m; m = m->next_symbol)
    {
	MCListBoxItem *mci = new MCListBoxItem;
	mci->insertString (m->name->ptr);
	mci->insertString (m->attr->str);
	table->insertQuickItem (mci);
    }
    table->updateRows ();
    table->setAutoUpdate (TRUE);
    table->repaint ();
}
	

void EditMacros::ok ()
{
    int i;
    QString name;
    QString str;

    discardMacros ();
    if (insertBtn->isEnabled())
	insert ();
    
    for (i = table->count()-1; i >= 0; i--)
    {
	name = table->at (i)->lookupString (0);
	str = table->at (i)->lookupString (1);

	fprintf (stderr, "Entering macro: %s = %s\n", (const char *) name,
		(const char *) str);
	enterMacro (name,
		    (char *)((const char *)str),
		    (text_type_t)str.contains ('#'));
    }
    close ();
    emit closedMacros (TRUE);
}

void EditMacros::cancel ()
{
    close ();
    emit closedMacros (FALSE);
}


void EditMacros::insert ()
{
    QString name (idEdit->text());
    QString text (expandEdit->text());
    MCListBoxItem *i;
    int index;
    if ((index = table->findItem (name, 0)) == -1)
    {
	i = new MCListBoxItem;
	i->insertString (name);
	i->insertString (text);
	table->insertItem (i, table->currentItem()+1);
    }
    else
    {
	i = table->at (index);	
	if (i != NULL)
	{
	    i->removeString (1);
	    i->insertString (text, 1);
	}
    }
    table->repaint ();	
    insertBtn->setEnabled (FALSE);
}

void EditMacros::del ()
{
    if (table->currentItem () == -1)
	return;

    if (!opt->getPreference (NO_SILLY_WARN))
    {
    if (QMessageBox::query ("Warning", "Really delete selected item?",
                             "Yes", "No"))
	{
	    table->removeItem (table->currentItem ());
	}
    }
    else
	table->removeItem (table->currentItem ());
}

void EditMacros::clear ()
{
    if (QMessageBox::query ("Warning", "This will delete all macros!",
			    "OK", "Cancel"))
    {
	idEdit->setText ("");
	expandEdit->setText ("");
	table->clear (FALSE);
    }
}


void EditMacros::resizeColumn (int column)
{
    title->setColumnWidth (table->autoColumnWidth (column), column);
    title->repaint ();
}


/* Show the selected macro in the editLine widgets */
void EditMacros::displayMacro (int row)
{
    MCListBoxItem *mci = table->at (row);

    if (mci != NULL)
    {
	if (insertBtn->isEnabled())
	    insert ();
	idEdit->setText (mci->lookupString (0));
	expandEdit->setText (mci->lookupString (1));
	insertBtn->setEnabled (FALSE);
    }
}
	
void EditMacros::edited (const char *)
{
    insertBtn->setEnabled (TRUE);
}

