/*
 * Interface dialog.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#include "Interface.h"
#include "Edit.h"
#include "MBMessageBox.h"

Options *theOptions;

#define Inherited QWidget

#define BARRACUDA_RC ".barracuda"

Interface::Interface (QWidget* parent, const char* name)
	: Inherited (parent, name), InterfaceData (this),
	  MenuBarData ()	
{
    extern char parserError[PARSER_ERROR_LEN+1];
    s = new Sort();
    mySortSpec = new SortSpec;
    mySortSpec->makeDefault();
    e = new Edit();
    em = new EditMacros ();
    p = new Preferences ();
    vt = new ViewTypes ();
    ss = new SimpleSearch ();
    pd = new PrintDlg ();
    k = new Keybindings ();
    sd = new Search ();
    title = new MultiLabel (this, "title");
    table = new MCListBox (this, "table");
    accel = new Shortcut (this, "accel");
    fs = new FontSelect ();
    infoTimer = new QTimer (this);
    searchString = QString ("");
    searchRef = new Reference (SUPER_REFERENCE);
    int i;

    for (i = FIRST_FIELD; i <= LAST_FIELD; i++)
	searchRef->setField (i, ".*", STRING, TRUE);
    searchRef->id = ".*";
    for (i = 0; i <= LAST_REFERENCE-FIRST_REFERENCE; i++)
	searchRefTypes[i] = TRUE;

    QString rc (getenv ("HOME"));
    rc += "/" BARRACUDA_RC;
    opt = new Options (LAST_COMMAND);
    if (!opt->loadFromFile ((char *)((const char *)rc)))
	setTimedInfo (parserError);
    else
	setTimedInfo ("Options successfully loaded.");
    theOptions = opt;
    QObject::connect (em, SIGNAL (closedMacros (bool)),
		      this, SLOT (closedMacros (bool)));
    
    QObject::connect (accel, SIGNAL (bufferChanged (const char *)),
		      infoField, SLOT (setText (const char *)));

    QObject::connect (title, SIGNAL (widthChanged (int, int)),
		      table, SLOT (setColumnWidth (int, int)));
    QObject::connect (table, SIGNAL (horizValueChanged (int)),
		      title, SLOT (setScrollOffset (int)));
    QObject::connect (table, SIGNAL (selected (int)),
		      this, SLOT (showEdit ()));
    QObject::connect (title, SIGNAL (sepDblClicked (int)),
		      this, SLOT (resizeColumn (int)));
    QObject::connect (title, SIGNAL (dblClicked (int)),
		      this, SLOT (resizeColumn (int)));
    QObject::connect (title, SIGNAL (indexChanged (int, int)),
		      this, SLOT (indexChanged (int, int)));
    QObject::connect (title, SIGNAL (scrollChanged (int)),
		      table, SLOT (setScrollX (int)));
    QObject::connect (title, SIGNAL (pressed (int, int)),
		      this, SLOT (showPopup (int, int)));

    QObject::connect (infoTimer, SIGNAL (timeout ()),
		      this, SLOT (clearInfoField ()));

    QObject::connect (fs, SIGNAL (newFont (const QFont &)),
		      this, SLOT (colFont (const QFont &)));

    table->setGeometry (5, 60, 390, 205);
    table->setHorizTracking (TRUE);
    table->setCutCellsV (TRUE);
    table->setDrawLast (TRUE);
    for (i = 0; i < opt->numColumns(); i++)
	table->insertColumn ();

    title->setGeometry (6, 30, 389, 25);
	title->setDrawLast (TRUE);
    title->setFrameStyle (QFrame::Panel | QFrame::Raised);
    for (i = 0; i < opt->numColumns(); i++)
	title->insertColumn (columnLabel (i));

    setMinimumSize (320, 200);

    loadBitmaps ();

    initPopup ();

    addTips ();
    connectMenus ();
    connectBtns ();
    connectShortcuts ();

    // Init databases
    normalDb = new Database (50);
    searchRes = NULL;
    currentView = NORMALDB;
    currentName = QString ("");
    currentDir = QString (".");
    showSearchBtn->setEnabled (FALSE);
    showNormalBtn->setEnabled (FALSE);

    // Init options
    updatePref ();
    updateShortcuts ();
    updateFields ();
    updateTypes ();
    updateColumns ();
    
    setCaption ("Barracuda - [Untitled]");
    initMacros ();
    updateFonts ();
}



Interface::~Interface()
{
    delete e;
    delete em;
    delete p;
    delete k;
    delete vt;
    delete ss;
    delete title;
    delete table;
    delete accel;
    delete normalDb;
    delete popup;
    delete fs;
    if (searchRes != NULL)
	delete searchRes;
}

/* setShortcut
 *
 * Sets a shortcut and updates the menu text.
 * Args: id (as specified in MenuBarData.h)
 *       keys, n (vector with the keys)
 * =>    TRUE (if the shortcut was ok)
 *       FALSE (otherwise)
 */

bool Interface::setShortcut (int id, int keys[], int n)
{
    QPopupMenu *menu;
    QString s;
    QString keyString;
    char *t;
    int i;
    
    if (n < 0)
	return FALSE;

    if (n != 0)
    {
	keyString = Shortcut::keyToString(keys[0]);
	for (i = 1; i < n; i++)
	    keyString += " " + Shortcut::keyToString (keys[i]);
    }
    
    if (id >= FIRST_COMMAND && id <= MAX_MENU_ID)
    {
	if ((menu = findId (id)) == NULL)
	    return FALSE;
	
	if ((t = getText (id)) == NULL)
	    return FALSE;

	s = QString (t);
    }

    if (!accel->changeItem (keys, n, id))
    {
	QString mesg ("The keysequence ");
	mesg += keyString + " conflicts with a previously defined sequence.";
	QMessageBox::message ("Error", mesg);
	return FALSE;
    }

    if (n != 0)
	s += "\t(" + keyString + ")";
    
    if (id >= FIRST_COMMAND && id <= MAX_MENU_ID)
	menu->changeItem ((const char *)s, id);

    return TRUE;
}  

/* Resizes the window */
void Interface::resizeEvent (QResizeEvent *)
{
    int lbh = height() - leftBtn->height()  
	- table->y() - infoField->height() - 15;
    int lbw = width()-10;
    int btny = table->y() + lbh + 5;
    
    title->resize (lbw, 25);
    table->resize (lbw, lbh);

    leftBtn->move (leftBtn->x(), btny);
    leftDblArrow->move (leftDblArrow->x(), btny);
    recBtn->move (recBtn->x(), btny);
    rightBtn->move (rightBtn->x(), btny);
    rightDblArrow->move (rightDblArrow->x(), btny);

    iSrchBtn->move (lbw - iSrchBtn->width(), btny);
    searchBtn->move (iSrchBtn->x() - 5 - searchBtn->width(), btny);
    rSrchBtn->move (searchBtn->x() - 5 - rSrchBtn->width(), btny);

    showSearchBtn->move (rSrchBtn->x() - 50, btny);
    showNormalBtn->move (showSearchBtn->x() - 5 - showNormalBtn->width(), 
			 btny);

    infoField->move (infoField->x(), btny + leftBtn->height() + 5);
    infoField->resize (lbw, infoField->height());
}

/* Sets the infoField when a menuItem is highlighted */
void Interface::setInfoField (int menuID)
{
    static const char *menuInfoTexts[] = {
	"Begin with a new database",
	"",
	"Open a database file",
	"Open a macro file",
	"Save this database",
	"Save this database with a new name",
	"Save the macros",
	"Print this database",
	"Exit from barracuda",
	"",
	"Delete the higlighted record",
	"Edit the highlighted record",
	"Edit/View all macros",
	"Set preferences",
	"Find a string in the current database",
	"Specify keybindings",
	"Specify what reference types to view",
	"Save the options to file",
	"Search for regular expression",
	"Sort the database",
	"Start a Web browser",
	"Show exiting information about the authors",
	"", "", "", "", "", "", "", "", "", "", "", "", ""};

    setTimedInfo (menuInfoTexts[menuID]);
}

/* setTimedInfo
 *
 * Sets the infoField to the given string and starts the timer
 * Args: text
 */

void Interface::setTimedInfo (const char *text)
{
    infoField->setText (text);
    infoTimer->start (INFO_TIMEOUT, TRUE);
}

/* setInfoField
 *
 * Sets the infoFiled to the given text
 */

void Interface::setInfoField (const char *text)
{
    infoField->setText (text);
}


/* clearInfoField
 *
 * Args: 
 * =>
 */

void Interface::clearInfoField ()
{
    infoField->setText ("");
}

/* loadBitmaps
 *
 * Args: none
 * => nothing
 * Loads the bitmaps for the buttons.
 */

void Interface::loadBitmaps ()
{
    /* Load record button pixmap */
    decorateButton (recBtn, "New", "/record.bmp");
    decorateButton (leftBtn, "<", "/back.bmp");
    decorateButton (rightBtn, ">", "/play.bmp");
    decorateButton (leftDblArrow, "<<", "/rev.bmp");
    decorateButton (rightDblArrow, ">>", "/ffw.bmp");
    decorateButton (iSrchBtn, "?>", "/rightsearch.bmp");
    decorateButton (searchBtn, "?", "/search.bmp");
    decorateButton (rSrchBtn, "<?", "/leftsearch.bmp");
    decorateButton (showSearchBtn, "S", "/showsearch_greyad.bmp");
    decorateButton (showNormalBtn, "N", "/shownormal_greyad.bmp");
}


/* decorateButton
 *
 * Args: button         - the button to decorate
 *       altText        - Alternative text if the image isn't found
 *       filename       - The filename to look for the image in
 * => nothing
 */
void Interface::decorateButton (
    QPushButton *button, 
    char *altText, 
    char *filename)
{
    QPixmap *pm = new QPixmap();
    QString path = getenv("BARRACUDAHOME");
    QString def_path = BARRACUDA_PATH;
    
    if (pm->load (path + "/bitmaps" + filename))
	button->setPixmap (*pm);
    else if (pm->load (def_path + "/bitmaps" + filename))
	button->setPixmap (*pm);
    else
	button->setText (altText);
    delete pm;
}

/* addTips
 *
 * Adds QToolTips to all buttons
 */

void Interface::addTips ()
{
    QToolTip::add (recBtn, "Insert new references");
    QToolTip::add (leftBtn, "Go back one reference");
    QToolTip::add (rightBtn, "Go forward one reference");
    QToolTip::add (leftDblArrow, "Go to the top");
    QToolTip::add (rightDblArrow, "Go to the bottom");
    QToolTip::add (iSrchBtn, "Simple search forward");
    QToolTip::add (searchBtn, "Simple search");
    QToolTip::add (rSrchBtn, "Simple search backward");
    QToolTip::add (showSearchBtn, "Show last search result");
    QToolTip::add (showNormalBtn, "Show normal database");
}
    
/* connectShortcuts 
 *
 * Connects the shortcuts to dummy keys, to make a keybinding
 * call accel->changeItem for that id.
 */

void Interface::connectShortcuts ()
{
    int i;
    int k[2];

    for (i = 0; i <= LAST_COMMAND; i++)
	accel->insertItem (k, 0, i);

    accel->connectItem (EXIT_MENU_ID, this, SLOT (quit ()));
    accel->connectItem (EDIT_RECORD_MENU_ID, this, SLOT (showEdit ()));
    accel->connectItem (ARTICLE_MENU_ID, this, SLOT (insertArticle ()));
    accel->connectItem (INPROCEEDINGS_MENU_ID, this, SLOT (insertInProceedings ()));
    accel->connectItem (INCOLLECTION_MENU_ID, this, SLOT (insertInCollection ()));
    accel->connectItem (INBOOK_MENU_ID, this, SLOT (insertInBook ()));
    accel->connectItem (BOOK_MENU_ID, this, SLOT (insertBook()));
    accel->connectItem (BOOKLET_MENU_ID, this, SLOT (insertBooklet()));
    accel->connectItem (PHDTHESIS_MENU_ID, this, SLOT (insertPhdThesis()));
    accel->connectItem (MASTERSTHESIS_MENU_ID, this, SLOT (insertMastersThesis ()));
    accel->connectItem (TECHREPORT_MENU_ID, this, SLOT (insertTechReport ()));
    accel->connectItem (MANUAL_MENU_ID, this, SLOT (insertManual ()));
    accel->connectItem (UNPUBLISHED_MENU_ID, this, SLOT (insertUnPublished ()));
    accel->connectItem (MISC_MENU_ID, this, SLOT (insertMisc ()));
    accel->connectItem (PROCEEDINGS_MENU_ID, this, SLOT (insertProceedings()));
    accel->connectItem (OPEN_DB_MENU_ID, this, SLOT (openDb()));
    accel->connectItem (OPEN_MACRO_MENU_ID, this, SLOT (openMacros()));
    accel->connectItem (SAVE_MACROS_MENU_ID, this, SLOT (saveMacros()));
    accel->connectItem (SAVE_MENU_ID, this, SLOT (saveDatabase ()));
    accel->connectItem (SAVE_AS_MENU_ID, this, SLOT (saveAsDb()));
    accel->connectItem (SAVE_OPTIONS_MENU_ID, this, SLOT (saveOptions ()));
    accel->connectItem (PREFERENCES_MENU_ID, this, SLOT (pref ()));
    accel->connectItem (VIEW_TYPES_MENU_ID, this, SLOT (types ()));
    accel->connectItem (NEW_MENU_ID, this, SLOT (newDb ()));
    accel->connectItem (PRINT_MENU_ID, this, SLOT (print ()));
    accel->connectItem (KEYBINDINGS_MENU_ID, this, SLOT (keybind ()));
    accel->connectItem (SPAWN_BROWSER_MENU_ID, this, SLOT (helpDlg ()));
    accel->connectItem (ABOUT_MENU_ID, this, SLOT (about ()));
    accel->connectItem (EDIT_MACRO_MENU_ID, this, SLOT (showEditMacros ()));
    accel->connectItem (SORT_MENU_ID, this, SLOT (sort ()));
    accel->connectItem (DELETE_RECORD_MENU_ID, this, SLOT (deleteRef ()));
    accel->connectItem (SIMPLE_SEARCH_MENU_ID, this, SLOT (simpleSearch ()));
    accel->connectItem (INSERT_RECORD_MENU_ID, this, SLOT (showInsert ()));
    accel->connectItem (SEARCH_MENU_ID, this, SLOT (search ()));

    accel->connectItem (REWIND_BTN_ID, this, SLOT (rewind ()));
    accel->connectItem (BACK_BTN_ID, this, SLOT (back ()));
    accel->connectItem (FORWARD_BTN_ID, this, SLOT (forward ()));
    accel->connectItem (FAST_FORWARD_BTN_ID, this, SLOT (fastForward ()));
    accel->connectItem (SEARCH_BACK_BTN_ID, this, SLOT (searchBack()));
    accel->connectItem (SEARCH_FORWARD_BTN_ID, this, SLOT (searchForward()));
    accel->connectItem (SHOW_SEARCH_BTN_ID, this, SLOT (showSearch ()));
    accel->connectItem (SHOW_NORMAL_BTN_ID, this, SLOT (showNormal ()));
}


/* connectMenus
 *
 * Connects the menu items to their slots and inserts the popup menus into
 * the menu bar, also connects the popup menus to the infoField.
 */

void Interface::connectMenus ()
{
    edit->connectItem (EDIT_RECORD_MENU_ID, this, SLOT(showEdit()));
    edit->connectItem (INSERT_RECORD_MENU_ID, this, SLOT(showInsert()));
    edit->connectItem (EDIT_MACRO_MENU_ID, this, SLOT (showEditMacros ()));
    edit->connectItem (DELETE_RECORD_MENU_ID, this, SLOT (deleteRef ()));
    rec_ins->connectItem (ARTICLE_MENU_ID, this, SLOT (insertArticle ()));
    rec_ins->connectItem (INPROCEEDINGS_MENU_ID, this, SLOT (insertInProceedings ()));
    rec_ins->connectItem (INCOLLECTION_MENU_ID, this, SLOT (insertInCollection ()));
    rec_ins->connectItem (INBOOK_MENU_ID, this, SLOT (insertInBook ()));
    rec_ins->connectItem (BOOK_MENU_ID, this, SLOT (insertBook()));
    rec_ins->connectItem (BOOKLET_MENU_ID, this, SLOT (insertBooklet()));
    rec_ins->connectItem (PHDTHESIS_MENU_ID, this, SLOT (insertPhdThesis()));
    rec_ins->connectItem (MASTERSTHESIS_MENU_ID, this, SLOT (insertMastersThesis ()));
    rec_ins->connectItem (TECHREPORT_MENU_ID, this, SLOT (insertTechReport ()));
    rec_ins->connectItem (MANUAL_MENU_ID, this, SLOT (insertManual ()));
    rec_ins->connectItem (UNPUBLISHED_MENU_ID, this, SLOT (insertUnPublished ()));
    rec_ins->connectItem (MISC_MENU_ID, this, SLOT (insertMisc ()));
    rec_ins->connectItem (PROCEEDINGS_MENU_ID, this, SLOT (insertProceedings()));    
    open->connectItem (OPEN_DB_MENU_ID, this, SLOT(openDb()));
    open->connectItem (OPEN_MACRO_MENU_ID, this, SLOT (openMacros()));
    file->connectItem (SAVE_MACROS_MENU_ID, this, SLOT (saveMacros()));
    file->connectItem (NEW_MENU_ID, this, SLOT (newDb ()));
    file->connectItem (SAVE_AS_MENU_ID, this, SLOT(saveAsDb()));
    file->connectItem (SAVE_MENU_ID, this, SLOT(saveDatabase()));
    file->connectItem (EXIT_MENU_ID, this, SLOT(quit()));
    file->connectItem (PRINT_MENU_ID, this, SLOT (print ()));
    option->connectItem (SAVE_OPTIONS_MENU_ID, this, SLOT (saveOptions ()));
    option->connectItem (PREFERENCES_MENU_ID, this, SLOT (pref ()));
    option->connectItem (VIEW_TYPES_MENU_ID, this, SLOT (types ()));
    option->connectItem (KEYBINDINGS_MENU_ID, this, SLOT (keybind ()));
    tools->connectItem (SEARCH_MENU_ID, this, SLOT (search ()));
    tools->connectItem (SORT_MENU_ID, this, SLOT (sort ()));
    tools->connectItem (SIMPLE_SEARCH_MENU_ID, this, SLOT (simpleSearch ()));
    help->connectItem (SPAWN_BROWSER_MENU_ID, this, SLOT (helpDlg ()));
    help->connectItem (ABOUT_MENU_ID, this, SLOT (about ()));
    

    menu->insertItem ("File", file);
    menu->insertItem ("Edit", edit);
    menu->insertItem ("Option", option);
    menu->insertItem ("Tools", tools);
    menu->insertSeparator ();
    menu->insertItem ("Help", help);

    QObject::connect (file, SIGNAL (highlighted (int)),
		      this, SLOT (setInfoField (int)));
    QObject::connect (edit, SIGNAL (highlighted (int)),
		      this, SLOT (setInfoField (int)));
    QObject::connect (option, SIGNAL (highlighted (int)),
		      this, SLOT (setInfoField (int)));
    QObject::connect (tools, SIGNAL (highlighted (int)),
		      this, SLOT (setInfoField (int)));
    QObject::connect (help, SIGNAL (highlighted (int)),
		      this, SLOT (setInfoField (int)));
    QObject::connect (rec_ins, SIGNAL (highlighted (int)),
		      this, SLOT (setInfoField (int)));
    QObject::connect (open, SIGNAL (highlighted (int)),
		      this, SLOT (setInfoField (int)));
}


/* connectBtns
 *
 * Connects the buttons to their slots.
 */

void Interface::connectBtns ()
{
    connect (leftDblArrow, SIGNAL (clicked ()), this, SLOT (rewind ()));
    connect (leftBtn, SIGNAL (clicked ()), this, SLOT (back ()));
    connect (recBtn, SIGNAL (clicked ()), this, SLOT (showInsert ()));
    connect (rightBtn, SIGNAL (clicked ()), this, SLOT (forward ()));
    connect (rightDblArrow, SIGNAL (clicked ()), this, SLOT (fastForward ()));

    connect (showNormalBtn, SIGNAL (clicked ()), this, SLOT (showNormal ()));
    connect (showSearchBtn, SIGNAL (clicked ()), this, SLOT (showSearch ()));
    
    connect (rSrchBtn, SIGNAL (clicked ()), this, SLOT (searchBack()));
    connect (searchBtn, SIGNAL (clicked ()), this, SLOT (simpleSearch()));
    connect (iSrchBtn, SIGNAL (clicked ()), this, SLOT (searchForward()));
    
}


/* columnLabel
 *
 * Args: col
 * => Label for the given column
 */

char *Interface::columnLabel (int col)
{
    char *res;
    int type;

    if ((type = opt->columnType (col)) == -1)
	return NULL;

    if ((res = strdup(keywords[type])) == NULL)
	return NULL;

    res[0] = toupper (res[0]);
    return res;
}


/* readDatabase
 *
 * Args: filename
 */

void Interface::readDatabase (const char *filename, bool show)
{
    extern char parserError[PARSER_ERROR_LEN+1];

    if (!normalDb->readFile (filename))
    {
	QMessageBox::message ("Error",  parserError, "Ok");
	clearInfoField ();
    }
    else
    {
	setCurrentName (filename);
	setTimedInfo (QString("Loaded file ")+QString (filename));
    }
    if (show)
	showDatabase (normalDb);
}

void Interface::saveDatabase (const char *filename)
{
    extern char parserError[PARSER_ERROR_LEN+1];

    if (!normalDb->saveFile (filename, FALSE))
    {
	QMessageBox::message ("Error", parserError, "Ok");
	clearInfoField ();
    }
    else
    {
	setTimedInfo (QString("Saved file ") + QString (filename));
	setCurrentName ((const char *) filename);
    }
}

void Interface::saveDatabase ()
{
    if (currentName.isEmpty())
	saveAsDb();
    else
	saveDatabase ((const char *)currentName);
}


/* discardDatabase
 *
 * Deletes the database and sees if it need saving.
 */

void Interface::discardDatabase (bool deleteRef)
{

    if (normalDb->isEmpty())
	return;

    if (normalDb->isSaved())
	normalDb->removeAll (deleteRef);
    else if (!opt->getPreference (NO_SILLY_WARN))
    {
	if (QMessageBox::query ("Save database", 
                              "Do you want to save the database?", 
                               "Yes","No"))       
	{
	    saveDatabase ();
	    normalDb->removeAll (deleteRef);
	}
	else
	    normalDb->removeAll (deleteRef);
    }
    else
	normalDb->removeAll (deleteRef);

    table->clear (FALSE);
    discardMacros ();
}
	

/* searchToNormal
 *
 * Makes the searchRes database the normal one and discards 
 * the normal database.
 * => TRUE if it really was done.
 */

bool Interface::searchToNormal (bool redisplay)
{
    bool doit = FALSE;

    if (!opt->getPreference (NO_SILLY_WARN))
    {
	if (QMessageBox::query ("Change normal database",
				"This will make the search result the normal\n"
				"database and discard the normal one. "
                 "Proceed?", "Yes", "No"))
	    doit = TRUE;
    }
    else
	doit = TRUE;

    if (doit)
    {
	discardDatabase (TRUE);
	normalDb = searchRes;
	currentView = NORMALDB;
	searchRes = NULL;
	if (redisplay)
	    showDatabase (normalDb);
	showNormalBtn->setEnabled (FALSE);
	showSearchBtn->setEnabled (FALSE);
	decorateButton (showNormalBtn, "N", "/shownormal_greyad.bmp");
	decorateButton (showSearchBtn, "S", "/showsearch_greyad.bmp");
    }

    return doit;
}


/* setCurrentName
 *
 * Sets the filename for the normal database.
 * Args: name
 */

void Interface::setCurrentName (const char *name, bool changeNormal)
{
    QString cap;
    currentName = QString (name);
    
    if (changeNormal)
	normalName = currentName;
    
    cap = QString ("Barracuda - [");
    
    if (currentName.isEmpty())
	cap += "Untitled]";
    else
	cap += currentName + "]";
    
    setCaption ((const char *)cap);
}


/* showDatabase
 *
 * Shows the given database in the table.
 * Args: db
 */

void Interface::showDatabase (Database *db)
{
    long i;

    table->setAutoUpdate (FALSE);
    table->clear (FALSE);
    for (i = 0; i <= db->maxIndex(); i++)
	showReference ((*db)[i]);
    table->updateRows ();
    table->setAutoUpdate (TRUE);
    table->repaint ();
}

/* showReference
 *
 * Adds a reference at the end of the table
 */

void Interface::showReference (Reference *r)
{
    MCListBoxItem *item = makeItem (r);
    
	table->insertQuickItem (item);
}


/* makeItem
 *
 * Args: r (The reference to make an item for)
 * => The MCListBoxItem for r
 */

MCListBoxItem *Interface::makeItem (Reference *r)
{
    MCListBoxItem *item = new MCListBoxItem (FALSE);
    CHECK_PTR (item);
    int i, t;
    MacroOrString *mos;
    const char* str;
    
    for (i = 0; i < opt->numColumns(); i++)
    {
	if ((t = opt->columnType (i)) != -1)
	{
	    if (t == ID)
		item->insertString (r->id);
	    else
	    {
		if ((mos = r->getField (t)) == NULL)
		    item->insertString ("");
		else
		{
		    if (opt->getPreference (EXPAND_MACRO))
			str = r->getField (t)->getString ();
		    else
			str = r->getField (t)->getMacro ();
		    
		    if (str != NULL)
			item->insertString (str);
		    else
			item->insertString ("");
		}
	    }
	}
    }
    return item;
}


/* update*
 *
 * Updates the options
 */

void Interface::updatePref ()
{
    extern int bibdebug;
    int cur = table->currentItem ();
        
    bibdebug = opt->getPreference (VERBOSE_PARSE);
    MacroOrString::setConvertTex (opt->getPreference (CONVERT_TEX));
    title->setDoDrag (!opt->getPreference (LAZY_COLUMN));
    table->setDelimiterStyle ((enum PenStyle) opt->getPreference (DEL_STYLE));
    if (currentView == NORMALDB)
	showDatabase (normalDb);
    else
	showDatabase (searchRes);
    table->setCurrentItem (cur);
}
    
void Interface::updateShortcuts ()
{
    shortcut **s;
    int n, i;

    if ((n = opt->getShortcuts (&s)) == 0)
	return;

    for (i = 0; i < n; i++)
	setShortcut (s[i]->id, s[i]->keys, s[i]->n);
}


void Interface::updateFields ()
{
    if (currentView == NORMALDB)
	showDatabase (normalDb);
    else
	showDatabase (searchRes);
}

void Interface::updateTypes ()
{
    int i;

    table->setAutoUpdate (FALSE);
    for (i = 0; i < opt->numColumns(); i++)
	if (opt->getType (opt->columnType (i)))
	{
	    table->setColumnVisible (TRUE, i);
	    title->setColumnVisible (TRUE, i);
	}
	else
	{
	    table->setColumnVisible (FALSE, i);
	    title->setColumnVisible (FALSE, i);
	}
    table->setAutoUpdate (TRUE);
    table->repaint ();
    resize (width (), height ());
}    


void Interface::updateColumns ()
{
    int i;

    for (i = 0; i < opt->numColumns (); i++)
	title->setColumnWidth (opt->getColumnWidth (opt->columnType (i)), i);
}

void Interface::updateFonts ()
{
    int i;

    table->setAutoUpdate (FALSE);
    for (i = 0; i < opt->numColumns (); i++)
    {
	title->setColumnFont (opt->getColumnFont (opt->columnType (i)), i);
	table->setColumnFont (opt->getColumnFont (opt->columnType (i)), i);
    }
    table->setAutoUpdate (TRUE);
    repaint ();
}

/* initPopup
 *
 * Init popup menu to be showed when a title is right clicked.
 */

void Interface::initPopup ()
{
    popup = new QPopupMenu;
    CHECK_PTR (popup);
    popup->insertItem ("Auto width", POPUP_AUTO_WIDTH);
    popup->insertItem ("Font", POPUP_FONT);
    popup->insertItem ("Hide", POPUP_HIDE);

    connect (popup, SIGNAL (activated (int)), this, SLOT (chosenPopup (int)));
    chosenCol = -1;
}
       

/* Show popup menu when a title is right clicked */
void Interface::showPopup (int col, int but)
{
    if (but != RightButton)
	return;
    
    chosenCol = col;
    popup->popup(cursor().pos());
}

/* Do something about the chosen item */
void Interface::chosenPopup (int id)
{
    switch (id)
    {
    case POPUP_AUTO_WIDTH:
	resizeColumn (chosenCol);
	break;
    case POPUP_HIDE:
	opt->setType (opt->columnType (chosenCol), FALSE);
	updateTypes ();
	break;
    case POPUP_FONT:
	fs->showFont (title->columnFont (chosenCol));
	fs->show ();
    }
}

/* colFont
 *
 * Sets chosenCol's font to the given one.
 * Args: Font reference
 */

void Interface::colFont (const QFont &f)
{
    opt->setColumnFont (opt->columnType (chosenCol), f);
    updateFonts ();
}
	
/* Show insert reference dialog */
void Interface::showInsert ()
{
    showInsert (ARTICLE);
}

void Interface::insertArticle ()
{
    showInsert (ARTICLE);
}

void Interface::insertInProceedings ()
{
    showInsert (INPROCEEDINGS);
}

void Interface::insertInCollection ()
{
    showInsert (INCOLLECTION);
}

void Interface::insertInBook ()
{
    showInsert (INBOOK);
}

void Interface::insertBook ()
{
    showInsert (BOOK);
}

void Interface::insertBooklet ()
{
    showInsert (BOOKLET);
}

void Interface::insertPhdThesis ()
{
    showInsert (PHDTHESIS);
}

void Interface::insertMastersThesis ()
{
    showInsert (MASTERSTHESIS);
}

void Interface::insertTechReport ()
{
    showInsert (TECHREPORT);
}

void Interface::insertManual ()
{
    showInsert (MANUAL);
}

void Interface::insertUnPublished ()
{
    showInsert (UNPUBLISHED);
}

void Interface::insertMisc ()
{
    showInsert (MISC);
}

void Interface::insertProceedings ()
{
    showInsert (PROCEEDINGS);
}



void Interface::showInsert (int ref_type)
{
    if (currentView == NORMALDB)
    {
	if (e->iShow (normalDb, ref_type, opt))
	{
	    showDatabase (normalDb);
	    normalDb->changed ();
	}
    }
    else
    {
	if (e->iShow (searchRes, ref_type, opt))
	    showDatabase (searchRes);
    }
}	


/* Show edit macros dialog */
void Interface::showEditMacros ()
{
    em->eShow (opt);
}

/* React to a closed macro edit box */
void Interface::closedMacros (bool ok)
{
    long i; 
    int j;
    MacroOrString *mos;
    bool skipAll = FALSE;
    
    if (ok)
    {
	for (i = 0; i <= normalDb->maxIndex(); i++)
	    for (j = 1; j < opt->numColumns(); j++)
	    {
		mos = (*normalDb)[i]->getField (opt->columnType (j));
		if (mos != NULL)
		{
		    if ((mos->type == MACRO) && 
			(lookupMacro (mos->getMacro()) == NULL))
		    {
			if (!opt->getPreference (NO_SILLY_WARN) && !skipAll)
			{
			    mos->type = STRING;
			    QString mesg ("The macro ");
			    mesg += mos->getMacro ();
			    mesg += " has been deleted.\n";
			    mesg += "All uses of it will be interpreted as "
				"a string.";
			    if (!QMessageBox::query ("Warning", mesg, 
						       "Ignore", "Ignore All"))
				skipAll = TRUE;
			}
		    }
		}
	    }
	if (currentView == NORMALDB)
	    showDatabase (normalDb);
	else
	    showDatabase (searchRes);
    }
}

/* Show edit reference dialog */
void Interface::showEdit()
{
    int index = table->currentItem();
    Reference *r;
    MCListBoxItem *i;
    
    if (currentView == NORMALDB)
	r = (*normalDb)[index];
    else
	r = (*searchRes)[index];

    if (r != NULL)
    {
        if (e->eShow (normalDb, r, opt))
	{
/*	    if (currentView == NORMALDB)
		showDatabase (normalDb);
	    else
		showDatabase (searchRes);
		*/	    
	    i = makeItem (r);
	    table->changeItem (i, index);
	    normalDb->changed ();
	}
    }
}


/* openDb
 *
 * Loads a database file.
 */

void Interface::openDb()
{
    bool ok;
    extern int replacePolicy;
    
    replacePolicy = 0;
    QString filename  = QFileDialog::getOpenFileName (currentDir, 
						      "*.bib", this);
    if (!filename.isEmpty())
    {
	currentDir = filename.left (filename.findRev (
	    '/', filename.length ()));
	if (normalDb->isEmpty())
	    readDatabase (filename);
	else
	{
	    ok = QMessageBox::query ("Merge files", "Do you want to merge "
                                 "this file with the shown database?",
                                 "Yes", "No");
	    if (!ok)
	    {
		discardDatabase ();
		readDatabase (filename);
	    }
	    else
	    {
		if (currentView == NORMALDB)
		    readDatabase (filename);
		else
		{
		    if (searchToNormal (FALSE))
			readDatabase (filename);
		    else
			return;
		}
	    }
	}
    }
    showDatabase (normalDb);
    currentView = NORMALDB;
}


/* openMacros
 *
 */

void Interface::openMacros ()
{
    extern int replacePolicy;
    extern char parserError[PARSER_ERROR_LEN+1];

    replacePolicy = 0;
    QString filename = QFileDialog::getOpenFileName (currentDir,
						     "*.macro", this);
    if (!filename.isEmpty ())
    {
	currentDir = filename.left (filename.findRev (
	    '/', filename.length ()));
	if (!normalDb->readFile ((const char *) filename))
	    QMessageBox::message ("Error", parserError);
	else
	    setTimedInfo (QString ("Loaded file ") + filename);
    }

    if (currentView == NORMALDB)
	showDatabase (normalDb);
    else
	showDatabase (searchRes);
}

/* saveMacros
 *
 */

void Interface::saveMacros ()
{
    extern char parserError[PARSER_ERROR_LEN+1];

    QString filename = QFileDialog::getSaveFileName (currentDir, 
						     "*.macro", this);
    if (!filename.isEmpty())
    {
	currentDir = filename.left (filename.findRev (
	    '/', filename.length ()));

	if (!normalDb->saveFile ((const char *) filename, TRUE))
	    setTimedInfo (parserError);
    }
}


/* saveAsDb
 *
 * Saves a database on file.
 */

void Interface::saveAsDb()
{
    if ((currentView == SEARCHRES && searchToNormal (TRUE)) ||
	(currentView == NORMALDB))
    {
	QString filename = QFileDialog::getSaveFileName (currentDir, 
							 "*.bib", this);
	if (!filename.isEmpty())
	{
	    currentDir = filename.left (filename.findRev (
		'/', filename.length ()));
	    saveDatabase ((const char *) filename);
	}
    }
}


/* quit
 *
 * Leaves the program
 */

void Interface::quit()
{
    if (!normalDb->isSaved())
    {
	switch (MBMessageBox::query ("Save changes", 
				     "Save changes to normal database?",
				     3, "Yes", "No", "Cancel"))
	{
	case 0:
	    saveDatabase ();
	    qApp->quit ();
	    break;
	case 1:
	    qApp->quit ();
	    break;
	case 2:
	    break;
	}
    }
    else
	qApp->quit ();
}


/* Show preference dialog */
void Interface::pref ()
{
    bool r = p->eShow (opt);

    if (r)
	updatePref ();
}

/* Show view types dialog */
void Interface::types ()
{
    if (vt->eShow (opt))
	updateTypes ();
}

/* Save options */
void Interface::saveOptions ()
{
    int i;
    QString rc (getenv ("HOME"));
    rc += "/" BARRACUDA_RC;
    
    for (i = 0; i < opt->numColumns (); i++)
    {
	opt->setColumnWidth (opt->columnType (i), 
			     title->columnWidth (i));
//	opt->setColumnFont (opt->columnType (i),
//			    title->columnFont (i));
    }
    opt->saveToFile ((char *)((const char *)rc));
    setTimedInfo ("Saved .barracuda");
    
}


/* Do the actual search */
void Interface::doSearch (searchDir dir)
{
    int cur, newcur;
    QString fail = QString ("Failed search");
    
    clearInfoField ();
    cur = table->currentItem ();
    if (cur == -1)
	cur = 0;
    
    if (dir == FORWARD)
	cur++;
    else if (dir == BACKWARD)
	cur--;
    
    if (currentView == NORMALDB)
	newcur = normalDb->search (searchString,
				   cur,
				   dir,
				   opt->getPreference (EXPAND_MACRO),
				   opt->getPreference (CASE_SEARCH));
    else
	newcur = searchRes->search (searchString,
				    cur,
				    dir,
				    opt->getPreference (EXPAND_MACRO),
				    opt->getPreference (CASE_SEARCH));
    if (newcur == -1)
	setTimedInfo (fail + " " + searchString);
    else
	table->setCurrentItem (newcur);
}


/* Search backwards */
void Interface::searchBack()
{
    searchDir dir = BACKWARD;

    if (searchString.isEmpty())
	if (!ss->getName (&dir, &searchString))
	    return;

    doSearch (dir);
}

/* Search forwards */
void Interface::searchForward()
{
    searchDir dir = FORWARD;

    if (searchString.isEmpty())
	if (!ss->getName (&dir, &searchString))
	    return;

    doSearch (dir);
}
    
/* Simple search dialog */
void Interface::simpleSearch ()
{
    searchDir dir = FORWARD;

    if (ss->getName (&dir, &searchString))
	doSearch (dir);
}


/* Empty the database */
void Interface::newDb ()
{
    if (searchRes != NULL)
    {
	searchRes->removeAll (TRUE);
	searchRes = NULL;
    }
    discardDatabase ();
    currentView = NORMALDB;
    setCurrentName ("");
}


void Interface::resizeColumn (int column)
{
    if ((currentView == NORMALDB && normalDb->isEmpty()) || 
	(currentView == SEARCHRES && searchRes == NULL))
	return;

    title->setColumnWidth (table->autoColumnWidth (column), column);
    title->repaint ();
}


/* Shows the print dialog */
void Interface::print ()
{
    char header[100];
    char tmpFile[100];
    extern char parserError[PARSER_ERROR_LEN+1];
    int pos;
    output_type_t res;
    bool pRef, pMacro;
    QString name;
    Database *db;

    // Choose which database to print
    if (currentView == NORMALDB)
	db = normalDb;
    else
	db = searchRes;
    
    // Return if nothing to be printed
    if (noMacros() && ((currentView == NORMALDB && normalDb->isEmpty()) ||
	(currentView == SEARCHRES && searchRes == NULL)))
	return;
    
    // Choose header for printed document
    if (currentView == NORMALDB && normalDb->isSaved())
	strcpy (header, currentName.right (
	    currentName.length() -
	    currentName.findRev ('/', currentName.length()) - 
	    1));
    else
	strcpy (header, "Current Database");
	    
    // Show dialog
    if (pd->eShow (&res, &name, &pRef, &pMacro))
    {
	sprintf (tmpFile, "barr%ld.bib", (long)getpid ());
	if (!db->saveFile (tmpFile, FALSE))
	    QMessageBox::message ("Error", parserError);
	else
	{
	    setInfoField ("Please wait while printing...");
	    infoField->repaint ();
	    tmpFile[strlen(tmpFile)-4] = '\0';
	    if (res == TEX)
			if ((pos = name.findRev (".tex", name.length(), FALSE)) != -1)
			    name.truncate (pos);
	    if (!::print (pMacro,
			  pRef,
			  res,
			  (char *)((const char *)name),
			  tmpFile,
			  header))
		QMessageBox::message ("Error", parserError);
	    // Remove temp file if option not TEX
	    if (res!=TEX) unlink (strcat (tmpFile, ".bib"));
	    clearInfoField ();
	}
    }
}


void Interface::rewind ()
{
    table->setCurrentItem (0);
}

void Interface::back ()
{
    table->setCurrentItem (table->currentItem() - 1);
}

void Interface::forward ()
{
    table->setCurrentItem (table->currentItem() + 1);
}

void Interface::fastForward ()
{
    table->setCurrentItem (table->count()-1);
}

/* Show keybindings dialog */
void Interface::keybind ()
{
    if (k->eShow (opt))
	updateShortcuts ();
}

/* Show help */
void Interface::helpDlg ()
{
    setInfoField ("Starting Netscape...");
    if (!helpTheUser ())
    {
	QString mesg = "Error while trying to open help.\n";
	mesg += "See " + getHelpUrl () + " for more info.";
    	QMessageBox::message ("Error", mesg);
    }
    clearInfoField ();
}

/* Show about */
void Interface::about ()
{
    QString mesg(512);

    mesg.sprintf(
		"Barracuda 1.0.5, Copyright (C) 1997-1999\n"
		"Magnus Christensson, Stefan Cronert, Stefan Hellkvist, Magnus Ljung.\n"
		"Maintained by Roland Krause <rokrau@pocus.wustl.edu>\n"
		"Barracuda comes with ABSOLUTELY NO WARRANTY. For details see COPYING\n\n"
		"There are currently %d references loaded",
		normalDb->maxIndex()+1);

    QMessageBox::message ("About", mesg);
}
			      

/* show sort dialog */
void Interface::sort ()
{
    if (s->eShow (mySortSpec))
    {
	setInfoField ("Sorting...");
	if (currentView == NORMALDB)
	{
	    normalDb->sort(mySortSpec, opt->getPreference (EXPAND_MACRO));
	    showDatabase (normalDb);
	}
	else
	{
	    searchRes->sort(mySortSpec, opt->getPreference (EXPAND_MACRO));
	    showDatabase (searchRes);
	}
	clearInfoField ();
    }
}


/* Shows the search dialog */
void Interface::search ()
{
    Database *res; 
    
    if (sd->eShow (searchRef, searchRefTypes))
    {
	setInfoField ("Searching...");
	if (currentView == NORMALDB)
	{
	    if (searchRes != NULL)
	    {
		searchRes->removeAll (TRUE);
		delete searchRes;
	    }
	    searchRes = normalDb->select (searchRef,
					  searchRefTypes,
					  opt->getPreference (EXPAND_MACRO),
					  opt->getPreference (CASE_SEARCH));
	}
	else
	{
	    res = searchRes->select (searchRef,
				     searchRefTypes,
				     opt->getPreference (EXPAND_MACRO),
				     opt->getPreference (CASE_SEARCH));
	    searchRes->removeAll (TRUE);
	    delete searchRes;
	    searchRes = res;
	}
	setCurrentName ("", FALSE);
	currentView = SEARCHRES;
	showDatabase (searchRes);
	showNormalBtn->setEnabled (TRUE);
	decorateButton (showNormalBtn, "N", "/shownormal.bmp");
	showSearchBtn->setEnabled (FALSE);
	decorateButton (showSearchBtn, "S", "/showsearch_greyad.bmp");
	clearInfoField ();
    }
}

/* SLOTS for the show normal and show search databases */
void Interface::showNormal ()
{
    if (!showNormalBtn->isEnabled ())
	return;
    if (currentView == SEARCHRES)
    {
	setCurrentName (normalName);
	showDatabase (normalDb);
	currentView = NORMALDB;
	showSearchBtn->setEnabled (TRUE);
	decorateButton (showSearchBtn, "S", "/showsearch.bmp");
	showNormalBtn->setEnabled (FALSE);
	decorateButton (showNormalBtn, "N", "/shownormal_greyad.bmp");
    }
}

void Interface::showSearch ()
{
    if (!showSearchBtn->isEnabled ())
	return;
    if (currentView == NORMALDB && searchRes != NULL)
    {
	setCurrentName ("", FALSE);
	showDatabase (searchRes);
	showSearchBtn->setEnabled (FALSE);
	decorateButton (showNormalBtn, "N", "/shownormal.bmp");
	decorateButton (showSearchBtn, "S", "/showsearch_greyad.bmp");
	showNormalBtn->setEnabled (TRUE);
	currentView = SEARCHRES;
    }
}
	

/* Deletes the current reference */
void Interface::deleteRef ()
{
    int cur = table->currentItem ();

    if (cur != -1)
    {
	if (!opt->getPreference (NO_SILLY_WARN))
	    if (!QMessageBox::query ("Warning", "Really delete highlighted "
                                 "reference?", "Yes", "No"))
		return;
	
	if (currentView == SEARCHRES)
	{
	    searchRes->removeReference (cur, TRUE);
	    searchRes->changed ();
	}
	else
	{
	    normalDb->removeReference (cur, TRUE);
	    normalDb->changed();
	}
	table->removeItem (cur);
    }
}


void Interface::indexChanged (int oldIndex, int newIndex)
{
    int i;

    if (oldIndex == newIndex)
	return;

    opt->setColumnIndex (opt->columnType (oldIndex), newIndex);
    table->setAutoUpdate (FALSE);
    for (i = 0; i < opt->numColumns (); i++)
    {
	table->setColumnWidth (title->columnWidth (i), i);
	table->setColumnVisible (title->columnVisible (i), i);
	table->setColumnFont (title->columnFont (i), i);
    }
    table->setAutoUpdate (TRUE);
    
    if (currentView == NORMALDB)
	showDatabase (normalDb);
    else
	showDatabase (searchRes);
}
