/*
 * Keybindings dialog.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */


#include "Keybindings.h"
#include <keywords.h>
#include <barracuda.h>
#include <qmsgbox.h>


#define Inherited QDialog

Keybindings::Keybindings(QWidget* parent, const char* name)
	:
	Inherited( parent, name, TRUE ),
	KeybindingsData( this )
{
    setCaption("Keybindings");
    
    recDlg = new Record ();
    connect(okButton, SIGNAL(clicked()), this, SLOT(ok()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(cancel()));
    connect(recBtn, SIGNAL (clicked ()), this, SLOT (rec ()));
    connect(comboBox, SIGNAL(activated(int)), 
	    this, SLOT(comboBoxClicked(int)));
    connect(edit, SIGNAL(returnPressed()), 
	    this, SLOT(editChanged()));
    connect(edit, SIGNAL(textChanged(const char*)), 
	    this, SLOT(whenChanged(const char*)));
 
    editChanged_ = FALSE;
}


Keybindings::~Keybindings()
{
    delete recDlg;
}



int idToIndexVect[] = {0, -1, 1, 2, 3, 4, 5, 6, 7, -1, 
		       21, 22, 23, 24, 29, 25, 26, 27, 28, 30, 31, 32,
		       8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
		       33, 34, 35, 36, 39, 40, 38, 37};

int indexToIdVect[] = {0, 2, 3, 4, 5, 6, 7, 8, 22, 23, 24, 25, 26, 
		       27, 28, 29, 30, 31, 32, 33, 34, 
		       10, 11, 12, 13, 15, 16, 17, 18, 14, 19, 20, 21,
		       35, 36, 37, 38, 42, 41, 39, 40};


int
indexToId(int i)
{
    return indexToIdVect[i];
}

int
idToIndex(int i)
{
    return idToIndexVect[i];
}


bool
Keybindings::eShow(Options *o)
{
    int id = o->getFirstDefinedId();

    myopt = new Options(o->maxShortcuts());
    myopt->assign(o);

    oldopt = o;
    
    edit->setText(o->getShortcutString(id));
    comboBox->setCurrentItem(idToIndex(id));
    
    editChanged_ = FALSE;
    indexThatChanged_ = -1;

    show();
    return result();
}

void
Keybindings::comboBoxClicked(int index)
{
    int id;

    if (editChanged_)
	editChanged();
    
    id = indexToId(index);
    edit->setText("");
    oldText = "";
    if (myopt->hasShortcut(id))
	edit->setText(myopt->getShortcutString(id));
    else
    	edit->setText("");
    

    editChanged_ = FALSE;
}

void
Keybindings::cancel()
{
    delete myopt;
    done(FALSE);
}

void
Keybindings::ok()
{
    if (editChanged_)
	editChanged();
    oldopt->assign(myopt);

    
    delete myopt;
    done(TRUE);
}

void 
Keybindings::rec ()
{
    edit->setText (recDlg->eShow ());
}

void
Keybindings::editChanged()
{
    QString newStr;
    shortcut *s = new shortcut;
    extern char parserError[PARSER_ERROR_LEN+1];
    bool parseShortcutString (char *, int, shortcut*);

    s->n = 0;
    
    newStr = oldText;
    
    if (indexThatChanged_ == -1)
	newStr = edit->text();

    if (parseShortcutString ((char *)((const char *) newStr), 1, s))
    {
	if (indexThatChanged_ != -1)
	    s->id = indexToId(indexThatChanged_);
	else
	    s->id = indexToId(comboBox->currentItem());

	if (!myopt->setShortcut(s))
	    QMessageBox::message ("Warning", 
				  "This shortcut conflicts "
				  "with an earlier one");
    }
    else
	QMessageBox::message ("Error", parserError);

    editChanged_ = FALSE;
    indexThatChanged_ = -1;
}


void
Keybindings::whenChanged(const char *s)
{
    indexThatChanged_ = comboBox->currentItem();
    editChanged_ = TRUE;
    oldText = s;
}




