/*
 * I/O module.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

#include <barracuda.h>
#include <db.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <Shortcut.h>
#include <qkeycode.h>

extern char parserError[PARSER_ERROR_LEN+1];

/* readFromFile
 *
 * Args: name of file to read, database to insert in
 * => 0 is bad
 */
int readFromFile (char* filename, Database &db)
{
  FILE* newFile;
  extern void bibrestart (FILE*);
  extern int currentLine;
  extern Database* currentDb;
  extern int bibparse ();
  extern int braceLevel;
  extern bool scanString;
  extern bool findAt;
  extern int referencesAdded;

  int ret;

  if (!(newFile = fopen (filename, "r")))
    {
      sprintf (parserError, "Error opening file %s: %s.",
	       filename, strerror (errno));
      return 0;
    }

  referencesAdded = 0;

  bibrestart (newFile);

  currentLine = 1;
  currentDb = &db;
  braceLevel = 0;
  scanString = 0;
  findAt = 1;
  ret = bibparse ();  

  fclose (newFile);
  if (ret == 1 && parserError[0] == '\0')
    ret = 0;
  return !ret;
}

/* saveToFile
 *
 * Args: filename, database, saveMacrosOnly
 * => 0 is bad
 */
int saveToFile (char* filename, Database& db, bool saveMacrosOnly)
{
  FILE* outFile;
  long maxIndex, i;

  if (!(outFile = fopen (filename, "w")))
    {
      sprintf (parserError, "Error opening file %s: %s.",
	       filename, strerror (errno));
      return 0;
    }

  maxIndex = db.maxIndex ();

  /* Mark used macros as referenced */
  unmarkAllMacros ();

  for (i = 0; i <= maxIndex; i++)
    for (int j = FIRST_FIELD; j <= LAST_FIELD; j++)
      if (db[i]->hasField (j) && db[i]->getField (j) && 
	  db[i]->getField (j)->type == MACRO)
	db[i]->getField (j)->getString ();

  printAllMacros (outFile);

  if (!saveMacrosOnly)
    {
      
      for (i = 0; i <= maxIndex; i++)
	{
	  db[i]->print (outFile);
	}
    }

  fclose (outFile);
  return 1;
}

/* loadOptionsFromFile
 *
 * Args: filename, options object
 * => 0 is bad
 */
int loadOptionsFromFile (char* filename, Options& ops)
{
  FILE* newFile;
  extern void optionrestart (FILE*);
  extern int optionCurrentLine;
  extern Options* options;
  extern int optionparse ();
  //  extern int optiondebug;

  int ret;

  if (!(newFile = fopen (filename, "r")))
    {
      sprintf (parserError, "Error opening file %s: %s.",
	       filename, strerror (errno));
      return 0;
    }

  optionrestart (newFile);

  optionCurrentLine = 1;
  options = &ops;
  // optiondebug = 1;
  ret = optionparse ();

  fclose (newFile);
  return !ret;
}

/* saveOptionsToFile
 *
 * Args: filename, options object
 * => 0 is bad
 */
int saveOptionsToFile (char* filename, Options& ops)
{
  FILE* outFile;
  int i, numberOfShortcuts;
  shortcut** s;

  if (!(outFile = fopen (filename, "w")))
    {
      sprintf (parserError, "Error opening file %s: %s.",
	       filename, strerror (errno));
      return 0;
    }

  for (i = FIRST_PREF; i <= LAST_PREF; i++)
    {
      if (!ops.defaultPreference (i, ops.getPreference (i)))
	fprintf (outFile, "(set-preference %s %d)\n",
		 optionLabels[i], ops.getPreference (i));
    }

  for (i = FIRST_REFERENCE; i <= LAST_REFERENCE; i++)
    {
      if (!ops.defaultField (i, ops.getField (i)))
	fprintf (outFile, "(show-reference %s %s)\n",
		 keywords[i], (ops.getField (i)) ? "true" : "false");
    }

  for (i = FIRST_FIELD; i <= LAST_FIELD+1; i++)
    {
      if (!ops.defaultType (i, ops.getType (i)))
	fprintf (outFile, "(show-attribute %s %s)\n",
		 keywords[i], (ops.getType (i)) ? "true" : "false");
    }

  for (i = FIRST_FIELD; i <= LAST_FIELD+1; i++)
    {
      if (!ops.defaultWidth (i, ops.getColumnWidth (i)))
	fprintf (outFile, "(set-column-width %s %d)\n",
		 keywords[i], ops.getColumnWidth (i));
    }

  for (i = FIRST_FIELD; i <= LAST_FIELD+1; i++)
    {
      if (!ops.defaultIndex (i, ops.columnIndex (i)))
	fprintf (outFile, "(set-column-index %s %d)\n",
		 keywords[i], ops.columnIndex (i));
    }

  for (i = FIRST_FIELD; i <= LAST_FIELD+1; i++)
    {
      if (!ops.defaultFont (i, ops.getColumnFont (i)))
	fprintf (outFile, "(set-column-font %s %s)\n",
		 keywords[i],
		 (const char*)externalFontRepresentation (ops.getColumnFont (i)));
    }

  numberOfShortcuts = ops.getShortcuts (&s);

  for (i = 0; i < numberOfShortcuts; i++)
    {
      if (!ops.defaultShortcut (s[i]))
	{
	  fprintf (outFile, "(set-shortcut \"");
	  for (int j = 0; j < s[i]->n; j++)
	    {
	      if (!j && !(s[i]->keys[j] & (SHIFT|CTRL|ALT)))
		fprintf (outFile, "%s",
			 (const char*) Shortcut::keyToString (s[i]->keys[j]));
	      else
		fprintf (outFile, "%c%s",
			 (s[i]->keys[j] & (SHIFT | CTRL | ALT)) ? '\\' : ' ',
			 (const char*) Shortcut::keyToString (s[i]->keys[j]));
	    }
	  fprintf (outFile, "\" %s)\n",
		   commands[s[i]->id]);
	}
    }

  fclose (outFile);
  return 1;
}

/* print
 *
 * Args: se below
 * => 0 is bad
 */
bool print (bool printMacros, bool printReferences, output_type_t outputType,
	    char* fileOrPrinterName, char* bibfilename, char* caption)
{
  FILE* writeToPipe;

  QString path = getenv("BARRACUDAHOME");
  QString def_path = BARRACUDA_PATH;
  QString commandLine;

  if (path.isEmpty ())
    commandLine = def_path;
  else
    commandLine = path;

  commandLine += "/print-script ";

  commandLine += (printMacros) ? "true " : "false ";
  commandLine += (printReferences) ? "true " : "false ";
  switch (outputType)
    {
    case TEX:
      commandLine += "tex ";
      break;
    case PS:
      commandLine += "ps ";
      break;
    case PRINTER:
      commandLine += "printer ";
    };
  commandLine += fileOrPrinterName;
  commandLine += " ";
  commandLine += bibfilename;
  commandLine += " \"";
  commandLine += caption;
  commandLine += "\"";

  /* Spawn print-script with the parameters */

  if (!(writeToPipe = popen ((const char*) commandLine, "w")))
    {
      sprintf (parserError, "Error creating printer job: %s.",
	       strerror (errno));
      return 0;
    }
  if (printMacros)
    {
      printMacrosForPrint (writeToPipe);
    }
  if (pclose (writeToPipe))
    {
      sprintf (parserError, "Error in print job.");
      return 0;
    }
  return 1;
}

/* spawnNetscape
 *
 * Args: url to open
 * => 0 is bad
 */
static bool spawnNetscape (QString url)
{
  FILE* fp;
  QString command("netscape -remote 'openURL(");
  
  /* Try to open a netscape with -remote */
  if (!(fp = popen (command + url + ")'" + " 2>&1 > /dev/null", "r")))
    return 0;
  if (pclose (fp))
    {
      /* -remote failed spawn a netscape */
      command = "netscape " + url + " 2>&1 > /dev/null" + " &";
      if (system (command) == -1)
	return 0;
    }
  return 1;
}

QString getHelpUrl ()
{
  QString ret;
  QString path = getenv("BARRACUDAHOME");
  QString def_path = BARRACUDA_PATH;

  ret = "file://localhost";

  if (path.isEmpty ())
    ret += def_path + "/html/barracuda_help.html";
  else
    ret += path + "/html/barracuda_help.html";

  return ret;
}

/* helpTheUser
 *
 * => 0 is bad
 */
bool helpTheUser (void)
{
  return spawnNetscape (getHelpUrl ());
}
