/*
 * Option-file parser.
 *
 * Barracuda 1.0.
 * Copyright (C) 1997 Magnus Christensson,
 *                    Stefan Cronert,
 *                    Stefan Hellkvist,
 *                    Magnus Ljung.
 *
 * Barracuda comes with ABSOLUTELY NO WARRANTY. For detailes see COPYING.
 */

%{

/* C++ declarations */
#include <barracuda.h>
#include <keywords.h>
#include <stdio.h>
#include <stdlib.h>
#include <options.h>
#include <Shortcut.h>

void optionerror (char* s);
Options* options;

extern char parserError[PARSER_ERROR_LEN+1];

typedef struct _semanticValue
{
  int type;
  char *str;
  int number;
  int line;
} semanticValue;

#define YYSTYPE semanticValue

int optionlex (YYSTYPE* yylval);

extern int optionCurrentLine;

%}

/* Bison declarations */

%token END_OF_FILE
%token Identifier String Number SetPreference SetShortcut
%token SetField SetType Boolean UndefShortcut SetColumnWidth
%token RPAREN SetColumnIndex SetColumnFont
%start OptionFile
%pure_parser

%%

/* Grammar */
OptionFile: OptOptionsPart END_OF_FILE { YYACCEPT; }
       ;

OptOptionsPart: /* empty */
              | OptOptionsPart Option
              ;
         
Option: '(' SetPreference Identifier Number RPAREN
{
  if ($3.type == 0)
    {
      sprintf (parserError, "Line %d: Illegal preference %s.",
	       $3.line, $3.str);
      free ($3.str);
      YYABORT;			/* Barf */
    }
  else if ($3.type < 0) /* command */
    {
      sprintf (parserError, "Line %d: Illegal preference %s.",
	       $3.line, commands[-$3.type-1]);
      YYABORT;			/* Barf */
    }
  else if ($3.type < FIRST_PREF || $3.type > LAST_PREF)
    {
      sprintf (parserError, "Line %d: Illegal preference %s.",
	       $3.line, keywords[$3.type]);
      YYABORT;			/* Barf */
    }
  else if (!options->setPreference ($3.type, $4.number))
    {
      sprintf (parserError, "Line %d: Illegal preference value %d.",
	       $4.line, $4.number);
      YYABORT;			/* Barf */
    }
}
      | '(' SetColumnWidth Identifier Number RPAREN
{
  /* Check that Identifier is a legal attribute */
  if ($3.type == 0)
    {
      sprintf (parserError, "Line %d: Illegal field %s.",
	       $3.line, $3.str);
      free ($3.str);
      YYABORT;
    }
  else if ($3.type < FIRST_FIELD || $3.type > LAST_FIELD+1)
    {
      sprintf (parserError, "Line %d: Illegal field %s.",
	       $3.line, 
	       ($3.type > 0) ? keywords[$3.type] : commands[-$3.type-1]);
      YYABORT;
    }

  /* Set corresponding option */
  else if (!options->setColumnWidth ($3.type, $4.number))
    {
      sprintf (parserError, "Line %d: Illegal column width %d.",
	       $4.line, $4.number);
      YYABORT;			/* Barf */
    }
}
      | '(' SetColumnFont Identifier Identifier RPAREN
{
  /* Check that Identifier is a legal attribute */
  if ($3.type == 0)
    {
      sprintf (parserError, "Line %d: Illegal field %s.",
	       $3.line, $3.str);
      free ($3.str);
      if (!$4.type) free ($4.str);

      YYABORT;
    }
  else if ($3.type < FIRST_FIELD || $3.type > LAST_FIELD+1)
    {
      sprintf (parserError, "Line %d: Illegal field %s.",
	       $3.line, 
	       ($3.type > 0) ? keywords[$3.type] : commands[-$3.type-1]);
      if (!$4.type) free ($4.str);

      YYABORT;
    }
  else if ($4.type != 0)
    {
      sprintf (parserError, "Line %d: Illegal font %s.",
	       $4.line,
	       ($4.type > 0) ? keywords[$4.type] : commands[-$4.type-1]);
      YYABORT;
    }

  /* Convert to internal format */
  {
    QFont font;
    /* Set corresponding option */

    if (!internalFontRepresentation (font, $4.str) ||
	!options->setColumnFont ($3.type, font))
      {
	sprintf (parserError, "Line %d: Illegal column font %s.",
		 $4.line, $4.str);
	free ($4.str);
	YYABORT;			/* Barf */
      }
  }
  free ($4.str);
}
      | '(' SetShortcut String Identifier RPAREN
{
  shortcut* s;

  s = new shortcut;

  /* Check that the Identifier is a command */
  if ($4.type == 0)
    {
      sprintf (parserError, "Line %d: Illegal command %s.",
	       $4.line, $4.str);
      free ($4.str);
      delete s;
      YYABORT;
    }
  else if ($4.type > 0) /* preference */
    {
      sprintf (parserError, "Line %d: Illegal command %s.",
	       $3.line, keywords[$4.type]);
       delete s;
      YYABORT;			/* Barf */
    }

  s->id = -$4.type-1;
  s->n = 0;

  /* Parse the shortcut string */
  if (!parseShortcutString ($3.str, $3.line, s))
    {
      delete s;
      YYABORT;
    }

  if (!options->setShortcut (s))
    {
      sprintf (parserError, "Line %d: Multiply defined shortcut.",
	       $2.line);
      delete s;
      YYABORT;
    }
}
      | '(' UndefShortcut String RPAREN
{
  shortcut s;

  /* Parse the shortcut string */
  if (!parseShortcutString ($3.str, $3.line, &s))
    {
      YYABORT;
    }

  if (!options->undefShortcut (&s))
    {
      sprintf (parserError, "Line %d: Error undeffing shortcut.",
	       $2.line);
      YYABORT;
    }
}
      | '(' UndefShortcut Identifier RPAREN
{
  /* Check that the Identifier is a command */
  if ($3.type == 0)
    {
      sprintf (parserError, "Line %d: Illegal command %s.",
	       $3.line, $3.str);
      free ($3.str);
      YYABORT;
    }
  else if ($3.type > 0) /* preference */
    {
      sprintf (parserError, "Line %d: Illegal command %s.",
	       $3.line, keywords[$3.type]);
      YYABORT;			/* Barf */
    }

  if (!options->undefShortcut (-$3.type-1))
    {
      sprintf (parserError, "Line %d: Error undeffing shortcut.",
	       $3.line);
      YYABORT;
    }
}
      | '(' SetField Identifier Boolean RPAREN
{
  /* Check that Identifier is a legal reference */
  if ($3.type == 0)
    {
      sprintf (parserError, "Line %d: Illegal reference type %s.",
	       $3.line, $3.str);
      free ($3.str);
      YYABORT;
    }
  else if ($3.type < FIRST_REFERENCE || $3.type > LAST_REFERENCE)
    {
      sprintf (parserError, "Line %d: Illegal reference type %s.",
	       $3.line, 
	       ($3.type > 0) ? keywords[$3.type] : commands[-$3.type-1]);
      YYABORT;
    }

  /* Set corresponding option */
  options->setField ($3.type, $4.type);
}
      | '(' SetColumnIndex Identifier Number RPAREN
{
  /* Check that Identifier is a legal attribute */
  if ($3.type == 0)
    {
      sprintf (parserError, "Line %d: Illegal field %s.",
	       $3.line, $3.str);
      free ($3.str);
      YYABORT;
    }
  else if ($3.type < FIRST_FIELD || $3.type > LAST_FIELD+1)
    {
      sprintf (parserError, "Line %d: Illegal field %s.",
	       $3.line, 
	       ($3.type > 0) ? keywords[$3.type] : commands[-$3.type-1]);
      YYABORT;
    }

  if (!options->setColumnIndex ($3.type, $4.number))
    {
      sprintf (parserError, "Line %d: Illegal column index value %d.",
	       $3.line, $4.number);
      YYABORT;
    }
}
      | '(' SetType Identifier Boolean RPAREN
{
  /* Check that Identifier is a legal attribute */
  if ($3.type == 0)
    {
      sprintf (parserError, "Line %d: Illegal field %s.",
	       $3.line, $3.str);
      free ($3.str);
      YYABORT;
    }
  else if ($3.type < FIRST_FIELD || $3.type > LAST_FIELD+1)
    {
      sprintf (parserError, "Line %d: Illegal field %s.",
	       $3.line, 
	       ($3.type > 0) ? keywords[$3.type] : commands[-$3.type-1]);
      YYABORT;
    }

  /* Set corresponding option */
  options->setType ($3.type, $4.type);
}
      | '(' error RPAREN
{
  sprintf (parserError, "Line %d: Parse error before ')'.",
	   $3.line);
  YYABORT;
}
      ;

%%

/* C++ code */
void optionerror (char*)
{
  sprintf (parserError, "Parse error.");
}

bool parseShortcutString (char* str, int line, shortcut* s)
{
  char* string = str;

  for (; string[0]; s->n++)
    {
      if (s->n > MAXSHORTCUT)
	{
	  sprintf (parserError, "Line %d: Ridiculously long shortcut.",
		   line);
	  return 0;
	}
      if (string[0] == '\\' || string[0] == ' ')
	string++;
      if ((s->keys[s->n] = stringToKey (string, &string)) == -1)
	{
	  sprintf (parserError, "Line %d: Illegal shortcut %s.",
		   line, str);
	  return 0;
	}      
    }

  return 1;
}
